@@ -1,13 +1,12 @@
 """
 novelWriter – Constants
 =======================
-Constants and maps for translating flags and enums to text
 
 File History:
 Created: 2019-04-28 [0.0.1]
 
 This file is a part of novelWriter
-Copyright 2018–2023, Veronica Berglyd Olsen
+Copyright 2018–2024, Veronica Berglyd Olsen
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -22,16 +21,18 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program. If not, see <https://www.gnu.org/licenses/>.
 """
+from __future__ import annotations
+
+import re
 
 from PyQt5.QtCore import QCoreApplication, QT_TRANSLATE_NOOP
 
-from novelwriter.enum import nwItemClass, nwItemLayout, nwOutline
+from novelwriter.enum import nwBuildFmt, nwItemClass, nwItemLayout, nwOutline
 
 
-def trConst(tString):
-    """Wrapper function for locally translating constants.
-    """
-    return QCoreApplication.translate("Constant", tString)
+def trConst(text: str) -> str:
+    """Wrapper function for locally translating constants."""
+    return QCoreApplication.translate("Constant", text)
 
 
 class nwConst:
@@ -41,9 +42,19 @@ class nwConst:
     FMT_FSTAMP = "%Y-%m-%d %H.%M.%S"  # FileName safe format
     FMT_DSTAMP = "%Y-%m-%d"           # Date only format
 
-    # Various Hard Limits
-    MAX_DOCSIZE   = 5000000   # Maxium size of a single document
-    MAX_BUILDSIZE = 10000000  # Maxium size of a project build
+    # URLs
+    URL_WEB     = "https://novelwriter.io"
+    URL_DOCS    = "https://docs.novelwriter.io"
+    URL_CODE    = "https://github.com/vkbo/novelWriter"
+    URL_REPORT  = "https://github.com/vkbo/novelWriter/issues"
+    URL_HELP    = "https://github.com/vkbo/novelWriter/discussions"
+    URL_RELEASE = "https://github.com/vkbo/novelWriter/releases/latest"
+
+    # Requests
+    USER_AGENT = "Mozilla/5.0 (compatible; novelWriter (Python))"
+
+    # Gui Settings
+    STATUS_MSG_TIMEOUT = 15000  # milliseconds
 
 # END Class nwConst
 
@@ -53,10 +64,33 @@ class nwRegEx:
     FMT_EI = r"(?<![\w\\])(_)(?![\s_])(.+?)(?<![\s\\])(\1)(?!\w)"
     FMT_EB = r"(?<![\w\\])([\*]{2})(?![\s\*])(.+?)(?<![\s\\])(\1)(?!\w)"
     FMT_ST = r"(?<![\w\\])([~]{2})(?![\s~])(.+?)(?<![\s\\])(\1)(?!\w)"
+    FMT_SC = r"(?i)(?<!\\)(\[[\/\!]?(?:i|b|s|u|sup|sub)\])"
+    FMT_SV = r"(?<!\\)(\[(?i)(?:fn|footnote):)(.+?)(?<!\\)(\])"
+
+    # Pre-Compiled RegEx
+    RX_SC = re.compile(FMT_SC)
 
 # END Class nwRegEx
 
 
+class nwShortcode:
+
+    BOLD_O   = "[b]"
+    BOLD_C   = "[/b]"
+    ITALIC_O = "[i]"
+    ITALIC_C = "[/i]"
+    STRIKE_O = "[s]"
+    STRIKE_C = "[/s]"
+    ULINE_O  = "[u]"
+    ULINE_C  = "[/u]"
+    SUP_O    = "[sup]"
+    SUP_C    = "[/sup]"
+    SUB_O    = "[sub]"
+    SUB_C    = "[/sub]"
+
+# END Class nwShortcode
+
+
 class nwHeaders:
 
     H_VALID = ("H0", "H1", "H2", "H3", "H4")
@@ -67,16 +101,22 @@ class nwHeaders:
 
 class nwFiles:
 
+    # Config Files
     CONF_FILE   = "novelwriter.conf"
+    RECENT_FILE = "recentProjects.json"
+
+    # Project Root Files
     PROJ_FILE   = "nwProject.nwx"
-    PROJ_DICT   = "wordlist.txt"
+    PROJ_BACKUP = "nwProject.bak"
     PROJ_LOCK   = "nwProject.lock"
     TOC_TXT     = "ToC.txt"
-    SESS_STATS  = "sessionStats.log"
-    INDEX_FILE  = "tagsIndex.json"
-    OPTS_FILE   = "guiOptions.json"
-    RECENT_FILE = "recentProjects.json"
-    BUILD_CACHE = "prevBuild.json"
+
+    # Project Meta Files
+    BUILDS_FILE = "builds.json"
+    INDEX_FILE  = "index.json"
+    OPTS_FILE   = "options.json"
+    DICT_FILE   = "userdict.json"
+    SESS_FILE   = "sessions.jsonl"
 
 # END Class nwFiles
 
@@ -116,6 +156,21 @@ class nwKeyWords:
 # END Class nwKeyWords
 
 
+class nwLists:
+
+    USER_CLASSES = [
+        nwItemClass.CHARACTER,
+        nwItemClass.PLOT,
+        nwItemClass.WORLD,
+        nwItemClass.TIMELINE,
+        nwItemClass.OBJECT,
+        nwItemClass.ENTITY,
+        nwItemClass.CUSTOM,
+    ]
+
+# END Class nwLists
+
+
 class nwLabels:
 
     CLASS_NAME = {
@@ -184,17 +239,79 @@ class nwLabels:
         nwOutline.FOCUS:  QT_TRANSLATE_NOOP("Constant", "Focus"),
         nwOutline.CHAR:   KEY_NAME[nwKeyWords.CHAR_KEY],
         nwOutline.PLOT:   KEY_NAME[nwKeyWords.PLOT_KEY],
-        nwOutline.TIME:   KEY_NAME[nwKeyWords.TIME_KEY],
         nwOutline.WORLD:  KEY_NAME[nwKeyWords.WORLD_KEY],
+        nwOutline.TIME:   KEY_NAME[nwKeyWords.TIME_KEY],
         nwOutline.OBJECT: KEY_NAME[nwKeyWords.OBJECT_KEY],
         nwOutline.ENTITY: KEY_NAME[nwKeyWords.ENTITY_KEY],
         nwOutline.CUSTOM: KEY_NAME[nwKeyWords.CUSTOM_KEY],
         nwOutline.SYNOP:  QT_TRANSLATE_NOOP("Constant", "Synopsis"),
     }
+    BUILD_FMT = {
+        nwBuildFmt.ODT:    QT_TRANSLATE_NOOP("Constant", "Open Document (.odt)"),
+        nwBuildFmt.FODT:   QT_TRANSLATE_NOOP("Constant", "Flat Open Document (.fodt)"),
+        nwBuildFmt.HTML:   QT_TRANSLATE_NOOP("Constant", "novelWriter HTML (.html)"),
+        nwBuildFmt.NWD:    QT_TRANSLATE_NOOP("Constant", "novelWriter Markup (.txt)"),
+        nwBuildFmt.STD_MD: QT_TRANSLATE_NOOP("Constant", "Standard Markdown (.md)"),
+        nwBuildFmt.EXT_MD: QT_TRANSLATE_NOOP("Constant", "Extended Markdown (.md)"),
+        nwBuildFmt.J_HTML: QT_TRANSLATE_NOOP("Constant", "JSON + novelWriter HTML (.json)"),
+        nwBuildFmt.J_NWD:  QT_TRANSLATE_NOOP("Constant", "JSON + novelWriter Markup (.json)"),
+    }
+    BUILD_EXT = {
+        nwBuildFmt.ODT:    ".odt",
+        nwBuildFmt.FODT:   ".fodt",
+        nwBuildFmt.HTML:   ".html",
+        nwBuildFmt.NWD:    ".txt",
+        nwBuildFmt.STD_MD: ".md",
+        nwBuildFmt.EXT_MD: ".md",
+        nwBuildFmt.J_HTML: ".json",
+        nwBuildFmt.J_NWD:  ".json",
+    }
+    UNIT_NAME = {
+        "mm": QT_TRANSLATE_NOOP("Constant", "Millimetres"),
+        "cm": QT_TRANSLATE_NOOP("Constant", "Centimetres"),
+        "in": QT_TRANSLATE_NOOP("Constant", "Inches"),
+    }
+    UNIT_SCALE = {
+        "mm": 1.0,
+        "cm": 10.0,
+        "in": 25.4,
+    }
+    PAPER_NAME = {
+        "A4":     QT_TRANSLATE_NOOP("Constant", "A4"),
+        "A5":     QT_TRANSLATE_NOOP("Constant", "A5"),
+        "A6":     QT_TRANSLATE_NOOP("Constant", "A6"),
+        "Legal":  QT_TRANSLATE_NOOP("Constant", "US Legal"),
+        "Letter": QT_TRANSLATE_NOOP("Constant", "US Letter"),
+        "Custom": QT_TRANSLATE_NOOP("Constant", "Custom"),
+    }
+    PAPER_SIZE = {
+        "A4":     (210.0, 297.0),
+        "A5":     (148.0, 210.0),
+        "A6":     (105.0, 148.0),
+        "Legal":  (215.9, 355.6),
+        "Letter": (215.9, 279.4),
+        "Custom": (-1.0, -1.0),
+    }
 
 # END Class nwLabels
 
 
+class nwHeadFmt:
+
+    BR      = "{BR}"
+    TITLE   = "{Title}"
+    CH_NUM  = "{Chapter}"
+    CH_WORD = "{Chapter:Word}"
+    CH_ROMU = "{Chapter:URoman}"
+    CH_ROML = "{Chapter:LRoman}"
+    SC_NUM  = "{Scene}"
+    SC_ABS  = "{Scene:Abs}"
+
+    ALL = [TITLE, CH_NUM, CH_WORD, CH_ROMU, CH_ROML, SC_NUM, SC_ABS]
+
+# END Class nwHeadFmt
+
+
 class nwQuotes:
     """Allowed quotation marks.
     Source: https://en.wikipedia.org/wiki/Quotation_mark
@@ -228,8 +345,7 @@ class nwQuotes:
 
 
 class nwUnicode:
-    """Supported unicode character constants and their HTML equivalents.
-    """
+    """Supported unicode character constants and their HTML equivalents."""
     # Unicode Constants
     # =================
 
@@ -283,7 +399,7 @@ class nwUnicode:
     U_PERMIL = "\u2030"  # Per mille sign
     U_DEGREE = "\u00b0"  # Degree symbol
     U_MINUS  = "\u2212"  # Minus sign
-    U_TIMES  = "\u00d7"  # Multiplaction sign
+    U_TIMES  = "\u00d7"  # Multiplication sign
     U_DIVIDE = "\u00f7"  # Division sign
 
     # Arrows
