@@ -1,13 +1,12 @@
 """
 novelWriter – GUI Project Details
 =================================
-Class holding the project details dialog
 
 File History:
-Created: 2021-01-03 [1.1rc1]
+Created: 2021-01-03 [1.1rc1] GuiProjectDetails
 
 This file is a part of novelWriter
-Copyright 2018–2023, Veronica Berglyd Olsen
+Copyright 2018–2024, Veronica Berglyd Olsen
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -22,106 +21,107 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program. If not, see <https://www.gnu.org/licenses/>.
 """
+from __future__ import annotations
 
 import math
 import logging
-import novelwriter
 
+from PyQt5.QtGui import QCloseEvent, QFont
 from PyQt5.QtCore import Qt, QSize, pyqtSlot
-from PyQt5.QtGui import QFont
 from PyQt5.QtWidgets import (
     QAbstractItemView, QDialogButtonBox, QGridLayout, QHBoxLayout, QLabel,
     QLineEdit, QSpinBox, QTreeWidget, QTreeWidgetItem, QVBoxLayout, QWidget
 )
 
+from novelwriter import CONFIG, SHARED
 from novelwriter.common import formatTime, numberToRoman
-from novelwriter.custom import PagedDialog, QSwitch
 from novelwriter.constants import nwUnicode
-from novelwriter.gui.components import NovelSelector
+from novelwriter.extensions.switch import NSwitch
+from novelwriter.extensions.pageddialog import NPagedDialog
+from novelwriter.extensions.novelselector import NovelSelector
 
 logger = logging.getLogger(__name__)
 
 
-class GuiProjectDetails(PagedDialog):
+class GuiProjectDetails(NPagedDialog):
 
-    def __init__(self, mainGui):
-        super().__init__(parent=mainGui)
+    def __init__(self, parent: QWidget) -> None:
+        super().__init__(parent=parent)
 
-        logger.debug("Initialising GuiProjectDetails ...")
+        logger.debug("Create: GuiProjectDetails")
         self.setObjectName("GuiProjectDetails")
 
-        self.mainConf   = novelwriter.CONFIG
-        self.mainGui    = mainGui
-        self.theProject = mainGui.theProject
-
         self.setWindowTitle(self.tr("Project Details"))
 
-        wW = self.mainConf.pxInt(600)
-        wH = self.mainConf.pxInt(400)
-        pOptions = self.theProject.options
+        wW = CONFIG.pxInt(600)
+        wH = CONFIG.pxInt(400)
+        pOptions = SHARED.project.options
 
         self.setMinimumWidth(wW)
         self.setMinimumHeight(wH)
         self.resize(
-            self.mainConf.pxInt(pOptions.getInt("GuiProjectDetails", "winWidth",  wW)),
-            self.mainConf.pxInt(pOptions.getInt("GuiProjectDetails", "winHeight", wH))
+            CONFIG.pxInt(pOptions.getInt("GuiProjectDetails", "winWidth",  wW)),
+            CONFIG.pxInt(pOptions.getInt("GuiProjectDetails", "winHeight", wH))
         )
 
-        self.tabMain = GuiProjectDetailsMain(self.mainGui, self.theProject)
-        self.tabContents = GuiProjectDetailsContents(self.mainGui, self.theProject)
+        self.tabMain = GuiProjectDetailsMain(self)
+        self.tabContents = GuiProjectDetailsContents(self)
 
         self.addTab(self.tabMain, self.tr("Overview"))
         self.addTab(self.tabContents, self.tr("Contents"))
 
         self.buttonBox = QDialogButtonBox(QDialogButtonBox.Close)
-        self.buttonBox.button(QDialogButtonBox.Close)
-        self.buttonBox.rejected.connect(self._doClose)
+        self.buttonBox.rejected.connect(self.close)
+        self.rejected.connect(self.close)
         self.addControls(self.buttonBox)
 
-        logger.debug("GuiProjectDetails initialisation complete")
+        logger.debug("Ready: GuiProjectDetails")
 
         return
 
-    def updateValues(self):
-        """Set all the values of the pages.
-        """
+    def __del__(self) -> None:  # pragma: no cover
+        logger.debug("Delete: GuiProjectDetails")
+        return
+
+    def updateValues(self) -> None:
+        """Set all the values of the pages."""
         self.tabMain.updateValues()
         self.tabContents.updateValues()
         return
 
     ##
-    #  Slots
+    #  Events
     ##
 
-    def _doClose(self):
-        """Save settings and close the dialog.
-        """
+    def closeEvent(self, event: QCloseEvent) -> None:
+        """Capture the close event and perform cleanup."""
         self._saveGuiSettings()
-        self.close()
+        event.accept()
+        self.deleteLater()
         return
 
     ##
     #  Internal Functions
     ##
 
-    def _saveGuiSettings(self):
-        """Save GUI settings.
-        """
-        winWidth  = self.mainConf.rpxInt(self.width())
-        winHeight = self.mainConf.rpxInt(self.height())
+    def _saveGuiSettings(self) -> None:
+        """Save GUI settings."""
+        winWidth  = CONFIG.rpxInt(self.width())
+        winHeight = CONFIG.rpxInt(self.height())
 
         cColWidth = self.tabContents.getColumnSizes()
-        widthCol0 = self.mainConf.rpxInt(cColWidth[0])
-        widthCol1 = self.mainConf.rpxInt(cColWidth[1])
-        widthCol2 = self.mainConf.rpxInt(cColWidth[2])
-        widthCol3 = self.mainConf.rpxInt(cColWidth[3])
-        widthCol4 = self.mainConf.rpxInt(cColWidth[4])
+        widthCol0 = CONFIG.rpxInt(cColWidth[0])
+        widthCol1 = CONFIG.rpxInt(cColWidth[1])
+        widthCol2 = CONFIG.rpxInt(cColWidth[2])
+        widthCol3 = CONFIG.rpxInt(cColWidth[3])
+        widthCol4 = CONFIG.rpxInt(cColWidth[4])
 
         wordsPerPage = self.tabContents.wpValue.value()
         countFrom    = self.tabContents.poValue.value()
         clearDouble  = self.tabContents.dblValue.isChecked()
 
-        pOptions = self.theProject.options
+        logger.debug("Saving State: GuiProjectDetails")
+        pOptions = SHARED.project.options
         pOptions.setValue("GuiProjectDetails", "winWidth",     winWidth)
         pOptions.setValue("GuiProjectDetails", "winHeight",    winHeight)
         pOptions.setValue("GuiProjectDetails", "widthCol0",    widthCol0)
@@ -140,18 +140,13 @@ class GuiProjectDetails(PagedDialog):
 
 class GuiProjectDetailsMain(QWidget):
 
-    def __init__(self, mainGui, theProject):
-        super().__init__(parent=mainGui)
+    def __init__(self, parent: QWidget) -> None:
+        super().__init__(parent=parent)
 
-        self.mainConf   = novelwriter.CONFIG
-        self.theProject = theProject
-        self.mainGui    = mainGui
-        self.mainTheme  = mainGui.mainTheme
-
-        fPx = self.mainTheme.fontPixelSize
-        fPt = self.mainTheme.fontPointSize
-        vPx = self.mainConf.pxInt(4)
-        hPx = self.mainConf.pxInt(12)
+        fPx = SHARED.theme.fontPixelSize
+        fPt = SHARED.theme.fontPointSize
+        vPx = CONFIG.pxInt(4)
+        hPx = CONFIG.pxInt(12)
 
         # Header
         # ======
@@ -241,25 +236,25 @@ class GuiProjectDetailsMain(QWidget):
 
         return
 
-    def updateValues(self):
-        """Set all the values.
-        """
-        pIndex = self.theProject.index
+    def updateValues(self) -> None:
+        """Set all the values."""
+        project = SHARED.project
+        pIndex = project.index
         hCounts = pIndex.getNovelTitleCounts()
         nwCount = pIndex.getNovelWordCount()
-        edTime = self.theProject.getCurrentEditTime()
+        edTime = project.currentEditTime
 
-        self.bookTitle.setText(self.theProject.data.title or self.theProject.data.name)
-        self.projName.setText(self.tr("Project: {0}").format(self.theProject.data.name))
-        self.bookAuthors.setText(self.tr("By {0}").format(self.theProject.data.author))
+        self.bookTitle.setText(project.data.title or project.data.name)
+        self.projName.setText(self.tr("Project: {0}").format(project.data.name))
+        self.bookAuthors.setText(self.tr("By {0}").format(project.data.author))
 
         self.wordCountVal.setText(f"{nwCount:n}")
         self.chapCountVal.setText(f"{hCounts[2]:n}")
         self.sceneCountVal.setText(f"{hCounts[3]:n}")
-        self.revCountVal.setText(f"{self.theProject.data.saveCount:n}")
+        self.revCountVal.setText(f"{project.data.saveCount:n}")
         self.editTimeVal.setText(formatTime(edTime))
 
-        self.projPathVal.setText(str(self.theProject.storage.storagePath))
+        self.projPathVal.setText(str(project.storage.storagePath))
 
         return
 
@@ -274,30 +269,25 @@ class GuiProjectDetailsContents(QWidget):
     C_PAGE  = 3
     C_PROG  = 4
 
-    def __init__(self, mainGui, theProject):
-        super().__init__(parent=mainGui)
-
-        self.mainConf   = novelwriter.CONFIG
-        self.theProject = theProject
-        self.mainGui    = mainGui
-        self.mainTheme  = mainGui.mainTheme
+    def __init__(self, parent: QWidget) -> None:
+        super().__init__(parent=parent)
 
         # Internal
         self._theToC = []
         self._currentRoot = None
 
-        iPx = self.mainTheme.baseIconSize
-        hPx = self.mainConf.pxInt(12)
-        vPx = self.mainConf.pxInt(4)
-        pOptions = self.theProject.options
+        iPx = SHARED.theme.baseIconSize
+        hPx = CONFIG.pxInt(12)
+        vPx = CONFIG.pxInt(4)
+        pOptions = SHARED.project.options
 
         # Header
         # ======
 
         self.tocLabel = QLabel("<b>%s</b>" % self.tr("Table of Contents"))
 
-        self.novelValue = NovelSelector(self, self.theProject, self.mainGui)
-        self.novelValue.setMinimumWidth(self.mainConf.pxInt(200))
+        self.novelValue = NovelSelector(self)
+        self.novelValue.setMinimumWidth(CONFIG.pxInt(200))
         self.novelValue.novelSelectionChanged.connect(self._novelValueChanged)
 
         self.headBox = QHBoxLayout()
@@ -322,20 +312,21 @@ class GuiProjectDetailsContents(QWidget):
         ])
 
         treeHeadItem = self.tocTree.headerItem()
-        treeHeadItem.setTextAlignment(self.C_WORDS, Qt.AlignRight)
-        treeHeadItem.setTextAlignment(self.C_PAGES, Qt.AlignRight)
-        treeHeadItem.setTextAlignment(self.C_PAGE,  Qt.AlignRight)
-        treeHeadItem.setTextAlignment(self.C_PROG,  Qt.AlignRight)
+        if treeHeadItem:
+            treeHeadItem.setTextAlignment(self.C_WORDS, Qt.AlignRight)
+            treeHeadItem.setTextAlignment(self.C_PAGES, Qt.AlignRight)
+            treeHeadItem.setTextAlignment(self.C_PAGE,  Qt.AlignRight)
+            treeHeadItem.setTextAlignment(self.C_PROG,  Qt.AlignRight)
 
         treeHeader = self.tocTree.header()
         treeHeader.setStretchLastSection(True)
         treeHeader.setMinimumSectionSize(hPx)
 
-        wCol0 = self.mainConf.pxInt(pOptions.getInt("GuiProjectDetails", "widthCol0", 200))
-        wCol1 = self.mainConf.pxInt(pOptions.getInt("GuiProjectDetails", "widthCol1", 60))
-        wCol2 = self.mainConf.pxInt(pOptions.getInt("GuiProjectDetails", "widthCol2", 60))
-        wCol3 = self.mainConf.pxInt(pOptions.getInt("GuiProjectDetails", "widthCol3", 60))
-        wCol4 = self.mainConf.pxInt(pOptions.getInt("GuiProjectDetails", "widthCol4", 90))
+        wCol0 = CONFIG.pxInt(pOptions.getInt("GuiProjectDetails", "widthCol0", 200))
+        wCol1 = CONFIG.pxInt(pOptions.getInt("GuiProjectDetails", "widthCol1", 60))
+        wCol2 = CONFIG.pxInt(pOptions.getInt("GuiProjectDetails", "widthCol2", 60))
+        wCol3 = CONFIG.pxInt(pOptions.getInt("GuiProjectDetails", "widthCol3", 60))
+        wCol4 = CONFIG.pxInt(pOptions.getInt("GuiProjectDetails", "widthCol4", 90))
 
         self.tocTree.setColumnWidth(0, wCol0)
         self.tocTree.setColumnWidth(1, wCol1)
@@ -349,7 +340,7 @@ class GuiProjectDetailsContents(QWidget):
 
         wordsPerPage = pOptions.getInt("GuiProjectDetails", "wordsPerPage", 350)
         countFrom    = pOptions.getInt("GuiProjectDetails", "countFrom", 1)
-        clearDouble  = pOptions.getInt("GuiProjectDetails", "clearDouble", True)
+        clearDouble  = pOptions.getBool("GuiProjectDetails", "clearDouble", True)
 
         wordsHelp = (
             self.tr("Typical word count for a 5 by 8 inch book page with 11 pt font is 350.")
@@ -386,7 +377,7 @@ class GuiProjectDetailsContents(QWidget):
         self.dblLabel = QLabel(self.tr("Clear double pages"))
         self.dblLabel.setToolTip(dblHelp)
 
-        self.dblValue = QSwitch(self, 2*iPx, iPx)
+        self.dblValue = NSwitch(self, 2*iPx, iPx)
         self.dblValue.setChecked(clearDouble)
         self.dblValue.setToolTip(dblHelp)
         self.dblValue.clicked.connect(self._populateTree)
@@ -414,9 +405,8 @@ class GuiProjectDetailsContents(QWidget):
 
         return
 
-    def getColumnSizes(self):
-        """Return the column widths for the tree columns.
-        """
+    def getColumnSizes(self) -> list[int]:
+        """Return the column widths for the tree columns."""
         retVals = [
             self.tocTree.columnWidth(0),
             self.tocTree.columnWidth(1),
@@ -426,26 +416,23 @@ class GuiProjectDetailsContents(QWidget):
         ]
         return retVals
 
-    def updateValues(self):
-        """Populate the tree.
-        """
+    def updateValues(self) -> None:
+        """Populate the tree."""
         self._currentRoot = None
         self.novelValue.updateList()
         self.novelValue.setHandle(self.novelValue.firstHandle)
         self._prepareData(self.novelValue.firstHandle)
         self._populateTree()
-
         return
 
     ##
     #  Internal Functions
     ##
 
-    def _prepareData(self, rootHandle):
-        """Extract the information from the project index.
-        """
+    def _prepareData(self, rootHandle: str | None) -> None:
+        """Extract the information from the project index."""
         logger.debug("Populating ToC from handle '%s'", rootHandle)
-        self._theToC = self.theProject.index.getTableOfContents(rootHandle, 2)
+        self._theToC = SHARED.project.index.getTableOfContents(rootHandle, 2)
         self._theToC.append(("", 0, self.tr("END"), 0))
         return
 
@@ -454,9 +441,8 @@ class GuiProjectDetailsContents(QWidget):
     ##
 
     @pyqtSlot(str)
-    def _novelValueChanged(self, tHandle):
-        """Refresh the tree with another root item.
-        """
+    def _novelValueChanged(self, tHandle: str) -> None:
+        """Refresh the tree with another root item."""
         if tHandle != self._currentRoot:
             self._prepareData(tHandle)
             self._populateTree()
@@ -464,9 +450,8 @@ class GuiProjectDetailsContents(QWidget):
         return
 
     @pyqtSlot()
-    def _populateTree(self):
-        """Set the content of the chapter/page tree.
-        """
+    def _populateTree(self) -> None:
+        """Set the content of the chapter/page tree."""
         dblPages = self.dblValue.isChecked()
         wpPage = self.wpValue.value()
         fstPage = self.poValue.value() - 1
@@ -498,7 +483,7 @@ class GuiProjectDetailsContents(QWidget):
                 progPage = f"{cPage:n}"
                 progText = f"{pgProg:.1f}{nwUnicode.U_THSP}%"
 
-            hDec = self.mainTheme.getHeaderDecoration(tLevel)
+            hDec = SHARED.theme.getHeaderDecoration(tLevel)
             if tTitle.strip() == "":
                 tTitle = self.tr("Untitled")
 
