@@ -1,13 +1,12 @@
 """
 novelWriter – GUI Project Settings
 ==================================
-GUI classes for the project settings dialog
 
 File History:
-Created: 2018-09-29 [0.0.1]
+Created: 2018-09-29 [0.0.1] GuiProjectSettings
 
 This file is a part of novelWriter
-Copyright 2018–2023, Veronica Berglyd Olsen
+Copyright 2018–2024, Veronica Berglyd Olsen
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -22,53 +21,59 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program. If not, see <https://www.gnu.org/licenses/>.
 """
+from __future__ import annotations
 
 import logging
-import novelwriter
 
-from PyQt5.QtGui import QIcon, QPixmap, QColor
-from PyQt5.QtCore import Qt, QLocale, pyqtSlot
+from typing import TYPE_CHECKING
+
+from PyQt5.QtGui import QCloseEvent, QIcon, QPixmap, QColor
+from PyQt5.QtCore import Qt, pyqtSignal, pyqtSlot
 from PyQt5.QtWidgets import (
     QColorDialog, QComboBox, QDialogButtonBox, QHBoxLayout, QLabel, QLineEdit,
-    QPushButton, QTreeWidget, QTreeWidgetItem, QVBoxLayout, QWidget
+    QPushButton, QTreeWidget, QTreeWidgetItem, QVBoxLayout, QWidget, qApp
 )
 
-from novelwriter.enum import nwAlert
+from novelwriter import CONFIG, SHARED
 from novelwriter.common import simplified
-from novelwriter.custom import QSwitch, PagedDialog, QConfigLayout
+from novelwriter.extensions.switch import NSwitch
+from novelwriter.extensions.pageddialog import NPagedDialog
+from novelwriter.extensions.configlayout import NConfigLayout
+
+if TYPE_CHECKING:  # pragma: no cover
+    from novelwriter.guimain import GuiMain
 
 logger = logging.getLogger(__name__)
 
 
-class GuiProjectSettings(PagedDialog):
+class GuiProjectSettings(NPagedDialog):
 
     TAB_MAIN    = 0
     TAB_STATUS  = 1
     TAB_IMPORT  = 2
     TAB_REPLACE = 3
 
-    def __init__(self, mainGui, focusTab=TAB_MAIN):
+    newProjectSettingsReady = pyqtSignal()
+
+    def __init__(self, mainGui: GuiMain, focusTab: int = TAB_MAIN) -> None:
         super().__init__(parent=mainGui)
 
-        logger.debug("Initialising GuiProjectSettings ...")
+        logger.debug("Create: GuiProjectSettings")
         self.setObjectName("GuiProjectSettings")
 
-        self.mainConf   = novelwriter.CONFIG
-        self.mainGui    = mainGui
-        self.theProject = mainGui.theProject
-
-        self.theProject.countStatus()
+        self.mainGui = mainGui
+        SHARED.project.countStatus()
         self.setWindowTitle(self.tr("Project Settings"))
 
-        wW = self.mainConf.pxInt(570)
-        wH = self.mainConf.pxInt(375)
-        pOptions = self.theProject.options
+        wW = CONFIG.pxInt(570)
+        wH = CONFIG.pxInt(375)
+        pOptions = SHARED.project.options
 
         self.setMinimumWidth(wW)
         self.setMinimumHeight(wH)
         self.resize(
-            self.mainConf.pxInt(pOptions.getInt("GuiProjectSettings", "winWidth",  wW)),
-            self.mainConf.pxInt(pOptions.getInt("GuiProjectSettings", "winHeight", wH))
+            CONFIG.pxInt(pOptions.getInt("GuiProjectSettings", "winWidth",  wW)),
+            CONFIG.pxInt(pOptions.getInt("GuiProjectSettings", "winHeight", wH))
         )
 
         self.tabMain    = GuiProjectEditMain(self)
@@ -83,78 +88,77 @@ class GuiProjectSettings(PagedDialog):
 
         self.buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
         self.buttonBox.accepted.connect(self._doSave)
-        self.buttonBox.rejected.connect(self._doClose)
+        self.buttonBox.rejected.connect(self.close)
+        self.rejected.connect(self.close)
         self.addControls(self.buttonBox)
 
-        # Flags
-        self._spellChanged = False
-
         # Focus Tab
         self._focusTab(focusTab)
 
-        logger.debug("GuiProjectSettings initialisation complete")
+        logger.debug("Ready: GuiProjectSettings")
 
         return
 
-    @property
-    def spellChanged(self):
-        return self._spellChanged
+    def __del__(self) -> None:  # pragma: no cover
+        logger.debug("Delete: GuiProjectSettings")
+        return
+
+    def closeEvent(self, event: QCloseEvent) -> None:
+        """Capture the close event and perform cleanup."""
+        self._saveGuiSettings()
+        event.accept()
+        self.deleteLater()
+        return
 
     ##
-    #  Slots
+    #  Private Slots
     ##
 
-    def _doSave(self):
-        """Save settings and close dialog.
-        """
+    @pyqtSlot()
+    def _doSave(self) -> None:
+        """Save settings and close dialog."""
+        project    = SHARED.project
         projName   = self.tabMain.editName.text()
         bookTitle  = self.tabMain.editTitle.text()
         bookAuthor = self.tabMain.editAuthor.text()
+        projLang   = self.tabMain.projLang.currentData()
         spellLang  = self.tabMain.spellLang.currentData()
         doBackup   = not self.tabMain.doBackup.isChecked()
 
-        self.theProject.data.setName(projName)
-        self.theProject.data.setTitle(bookTitle)
-        self.theProject.data.setAuthor(bookAuthor)
-        self.theProject.data.setDoBackup(doBackup)
-
-        # Remember this as updating spell dictionary can be expensive
-        self._spellChanged = self.theProject.data.setSpellLang(spellLang)
+        project.data.setName(projName)
+        project.data.setTitle(bookTitle)
+        project.data.setAuthor(bookAuthor)
+        project.data.setDoBackup(doBackup)
+        project.data.setSpellLang(spellLang)
+        project.setProjectLang(projLang)
 
         if self.tabStatus.colChanged:
             newList, delList = self.tabStatus.getNewList()
-            self.theProject.setStatusColours(newList, delList)
+            project.setStatusColours(newList, delList)
 
         if self.tabImport.colChanged:
             newList, delList = self.tabImport.getNewList()
-            self.theProject.setImportColours(newList, delList)
+            project.setImportColours(newList, delList)
 
         if self.tabStatus.colChanged or self.tabImport.colChanged:
             self.mainGui.rebuildTrees()
 
         if self.tabReplace.arChanged:
             newList = self.tabReplace.getNewList()
-            self.theProject.data.setAutoReplace(newList)
+            project.data.setAutoReplace(newList)
 
-        self._saveGuiSettings()
-        self.accept()
+        self.newProjectSettingsReady.emit()
+        qApp.processEvents()
+        self.close()
 
         return
 
-    def _doClose(self):
-        """Save settings and close the dialog.
-        """
-        self._saveGuiSettings()
-        self.reject()
-        return
-
     ##
     #  Internal Functions
     ##
 
-    def _focusTab(self, tab):
-        """Change which is the focused tab.
-        """
+    def _focusTab(self, tab: int) -> None:
+        """Change which is the focused tab."""
         if tab == self.TAB_MAIN:
             self.setCurrentWidget(self.tabMain)
         elif tab == self.TAB_STATUS:
@@ -165,16 +169,16 @@ class GuiProjectSettings(PagedDialog):
             self.setCurrentWidget(self.tabReplace)
         return
 
-    def _saveGuiSettings(self):
-        """Save GUI settings.
-        """
-        winWidth    = self.mainConf.rpxInt(self.width())
-        winHeight   = self.mainConf.rpxInt(self.height())
-        replaceColW = self.mainConf.rpxInt(self.tabReplace.listBox.columnWidth(0))
-        statusColW  = self.mainConf.rpxInt(self.tabStatus.listBox.columnWidth(0))
-        importColW  = self.mainConf.rpxInt(self.tabImport.listBox.columnWidth(0))
+    def _saveGuiSettings(self) -> None:
+        """Save GUI settings."""
+        winWidth    = CONFIG.rpxInt(self.width())
+        winHeight   = CONFIG.rpxInt(self.height())
+        replaceColW = CONFIG.rpxInt(self.tabReplace.listBox.columnWidth(0))
+        statusColW  = CONFIG.rpxInt(self.tabStatus.listBox.columnWidth(0))
+        importColW  = CONFIG.rpxInt(self.tabImport.listBox.columnWidth(0))
 
-        pOptions = self.theProject.options
+        logger.debug("Saving State: GuiProjectSettings")
+        pOptions = SHARED.project.options
         pOptions.setValue("GuiProjectSettings", "winWidth",    winWidth)
         pOptions.setValue("GuiProjectSettings", "winHeight",   winHeight)
         pOptions.setValue("GuiProjectSettings", "replaceColW", replaceColW)
@@ -188,26 +192,23 @@ class GuiProjectSettings(PagedDialog):
 
 class GuiProjectEditMain(QWidget):
 
-    def __init__(self, projGui):
-        super().__init__(parent=projGui)
-
-        self.mainConf   = novelwriter.CONFIG
-        self.mainGui    = projGui.mainGui
-        self.theProject = projGui.theProject
+    def __init__(self, parent: QWidget) -> None:
+        super().__init__(parent=parent)
 
         # The Form
-        self.mainForm = QConfigLayout()
-        self.mainForm.setHelpTextStyle(self.mainGui.mainTheme.helpText)
+        self.mainForm = NConfigLayout()
+        self.mainForm.setHelpTextStyle(SHARED.theme.helpText)
         self.setLayout(self.mainForm)
 
         self.mainForm.addGroupLabel(self.tr("Project Settings"))
 
-        xW = self.mainConf.pxInt(250)
+        xW = CONFIG.pxInt(250)
+        pData = SHARED.project.data
 
         self.editName = QLineEdit()
         self.editName.setMaxLength(200)
         self.editName.setMaximumWidth(xW)
-        self.editName.setText(self.theProject.data.name)
+        self.editName.setText(pData.name)
         self.mainForm.addRow(
             self.tr("Project name"),
             self.editName,
@@ -217,7 +218,7 @@ class GuiProjectEditMain(QWidget):
         self.editTitle = QLineEdit()
         self.editTitle.setMaxLength(200)
         self.editTitle.setMaximumWidth(xW)
-        self.editTitle.setText(self.theProject.data.title)
+        self.editTitle.setText(pData.title)
         self.mainForm.addRow(
             self.tr("Novel title"),
             self.editTitle,
@@ -227,36 +228,51 @@ class GuiProjectEditMain(QWidget):
         self.editAuthor = QLineEdit()
         self.editAuthor.setMaxLength(200)
         self.editAuthor.setMaximumWidth(xW)
-        self.editAuthor.setText(self.theProject.data.author)
+        self.editAuthor.setText(pData.author)
         self.mainForm.addRow(
             self.tr("Author(s)"),
             self.editAuthor,
             self.tr("Change whenever you want!")
         )
 
+        self.projLang = QComboBox(self)
+        self.projLang.setMaximumWidth(xW)
+        for tag, language in CONFIG.listLanguages(CONFIG.LANG_PROJ):
+            self.projLang.addItem(language, tag)
+        self.mainForm.addRow(
+            self.tr("Project language"),
+            self.projLang,
+            self.tr("Used when building the manuscript.")
+        )
+
+        langIdx = 0
+        if pData.language is not None:
+            langIdx = self.projLang.findData(pData.language)
+        if langIdx == -1:
+            langIdx = self.projLang.findData("en_GB")
+        if langIdx != -1:
+            self.projLang.setCurrentIndex(langIdx)
+
         self.spellLang = QComboBox(self)
         self.spellLang.setMaximumWidth(xW)
         self.spellLang.addItem(self.tr("Default"), "None")
-        langAvail = self.mainGui.docEditor.spEnchant.listDictionaries()
-        for spTag, spProv in langAvail:
-            qLocal = QLocale(spTag)
-            spLang = qLocal.nativeLanguageName().title()
-            self.spellLang.addItem("%s [%s]" % (spLang, spProv), spTag)
-
+        if CONFIG.hasEnchant:
+            for tag, language in SHARED.spelling.listDictionaries():
+                self.spellLang.addItem(language, tag)
         self.mainForm.addRow(
             self.tr("Spell check language"),
             self.spellLang,
             self.tr("Overrides main preferences.")
         )
 
-        spellIdx = 0
-        if self.theProject.data.spellLang is not None:
-            spellIdx = self.spellLang.findData(self.theProject.data.spellLang)
-        if spellIdx != -1:
-            self.spellLang.setCurrentIndex(spellIdx)
+        langIdx = 0
+        if pData.spellLang is not None:
+            langIdx = self.spellLang.findData(pData.spellLang)
+        if langIdx != -1:
+            self.spellLang.setCurrentIndex(langIdx)
 
-        self.doBackup = QSwitch(self)
-        self.doBackup.setChecked(not self.theProject.data.doBackup)
+        self.doBackup = NSwitch(self)
+        self.doBackup.setChecked(not pData.doBackup)
         self.mainForm.addRow(
             self.tr("No backup on close"),
             self.doBackup,
@@ -273,36 +289,31 @@ class GuiProjectEditStatus(QWidget):
     COL_LABEL = 0
     COL_USAGE = 1
 
-    KEY_ROLE = Qt.UserRole
-    COL_ROLE = Qt.UserRole + 1
-    NUM_ROLE = Qt.UserRole + 2
+    KEY_ROLE = Qt.ItemDataRole.UserRole
+    COL_ROLE = Qt.ItemDataRole.UserRole + 1
+    NUM_ROLE = Qt.ItemDataRole.UserRole + 2
 
-    def __init__(self, projGui, isStatus):
-        super().__init__(parent=projGui)
-
-        self.mainConf   = novelwriter.CONFIG
-        self.mainGui    = projGui.mainGui
-        self.theProject = projGui.theProject
-        self.mainTheme  = projGui.mainGui.mainTheme
+    def __init__(self, parent: QWidget, isStatus: bool) -> None:
+        super().__init__(parent=parent)
 
         if isStatus:
-            self.theStatus = self.theProject.data.itemStatus
+            self.theStatus = SHARED.project.data.itemStatus
             pageLabel = self.tr("Novel File Status Levels")
             colSetting = "statusColW"
         else:
-            self.theStatus = self.theProject.data.itemImport
+            self.theStatus = SHARED.project.data.itemImport
             pageLabel = self.tr("Note File Importance Levels")
             colSetting = "importColW"
 
-        wCol0 = self.mainConf.pxInt(
-            self.theProject.options.getInt("GuiProjectSettings", colSetting, 130)
+        wCol0 = CONFIG.pxInt(
+            SHARED.project.options.getInt("GuiProjectSettings", colSetting, 130)
         )
 
         self.colDeleted = []
         self.colChanged = False
         self.selColour = QColor(100, 100, 100)
 
-        self.iPx = self.mainTheme.baseIconSize
+        self.iPx = SHARED.theme.baseIconSize
 
         # The List
         # ========
@@ -321,16 +332,16 @@ class GuiProjectEditStatus(QWidget):
         # List Controls
         # =============
 
-        self.addButton = QPushButton(self.mainTheme.getIcon("add"), "")
+        self.addButton = QPushButton(SHARED.theme.getIcon("add"), "")
         self.addButton.clicked.connect(self._newItem)
 
-        self.delButton = QPushButton(self.mainTheme.getIcon("remove"), "")
+        self.delButton = QPushButton(SHARED.theme.getIcon("remove"), "")
         self.delButton.clicked.connect(self._delItem)
 
-        self.upButton = QPushButton(self.mainTheme.getIcon("up"), "")
+        self.upButton = QPushButton(SHARED.theme.getIcon("up"), "")
         self.upButton.clicked.connect(lambda: self._moveItem(-1))
 
-        self.dnButton = QPushButton(self.mainTheme.getIcon("down"), "")
+        self.dnButton = QPushButton(SHARED.theme.getIcon("down"), "")
         self.dnButton.clicked.connect(lambda: self._moveItem(1))
 
         # Edit Form
@@ -383,9 +394,8 @@ class GuiProjectEditStatus(QWidget):
 
         return
 
-    def getNewList(self):
-        """Return list of entries.
-        """
+    def getNewList(self) -> tuple[list, list]:
+        """Return list of entries."""
         if self.colChanged:
             newList = []
             for n in range(self.listBox.topLevelItemCount()):
@@ -405,9 +415,8 @@ class GuiProjectEditStatus(QWidget):
     ##
 
     @pyqtSlot()
-    def _selectColour(self):
-        """Open a dialog to select the status icon colour.
-        """
+    def _selectColour(self) -> None:
+        """Open a dialog to select the status icon colour."""
         if self.selColour is not None:
             newCol = QColorDialog.getColor(
                 self.selColour, self, self.tr("Select Colour")
@@ -421,24 +430,20 @@ class GuiProjectEditStatus(QWidget):
         return
 
     @pyqtSlot()
-    def _newItem(self):
-        """Create a new status item.
-        """
+    def _newItem(self) -> None:
+        """Create a new status item."""
         self._addItem(None, self.tr("New Item"), (100, 100, 100), 0)
         self.colChanged = True
         return
 
     @pyqtSlot()
-    def _delItem(self):
-        """Delete a status item.
-        """
+    def _delItem(self) -> None:
+        """Delete a status item."""
         selItem = self._getSelectedItem()
         if isinstance(selItem, QTreeWidgetItem):
             iRow = self.listBox.indexOfTopLevelItem(selItem)
             if selItem.data(self.COL_LABEL, self.NUM_ROLE) > 0:
-                self.mainGui.makeAlert(self.tr(
-                    "Cannot delete a status item that is in use."
-                ), nwAlert.ERROR)
+                SHARED.error(self.tr("Cannot delete a status item that is in use."))
             else:
                 self.listBox.takeTopLevelItem(iRow)
                 self.colDeleted.append(selItem.data(self.COL_LABEL, self.KEY_ROLE))
@@ -446,9 +451,8 @@ class GuiProjectEditStatus(QWidget):
         return
 
     @pyqtSlot()
-    def _saveItem(self):
-        """Save changes made to a status item.
-        """
+    def _saveItem(self) -> None:
+        """Save changes made to a status item."""
         selItem = self._getSelectedItem()
         if isinstance(selItem, QTreeWidgetItem):
             selItem.setText(self.COL_LABEL, simplified(self.editName.text()))
@@ -457,11 +461,10 @@ class GuiProjectEditStatus(QWidget):
                 self.selColour.red(), self.selColour.green(), self.selColour.blue()
             ))
             self.colChanged = True
-
         return
 
     @pyqtSlot()
-    def _selectedItem(self):
+    def _selectedItem(self) -> None:
         """Extract the info of a selected item and populate the settings
         boxes and button. If no item is selected, clear the form.
         """
@@ -469,7 +472,6 @@ class GuiProjectEditStatus(QWidget):
         if isinstance(selItem, QTreeWidgetItem):
             cols = selItem.data(self.COL_LABEL, self.COL_ROLE)
             name = selItem.text(self.COL_LABEL)
-
             pixmap = QPixmap(self.iPx, self.iPx)
             pixmap.fill(QColor(*cols))
             self.selColour = QColor(*cols)
@@ -477,31 +479,27 @@ class GuiProjectEditStatus(QWidget):
             self.colButton.setIcon(QIcon(pixmap))
             self.editName.selectAll()
             self.editName.setFocus()
-
             self.editName.setEnabled(True)
             self.colButton.setEnabled(True)
             self.saveButton.setEnabled(True)
-
         else:
             pixmap = QPixmap(self.iPx, self.iPx)
             pixmap.fill(QColor(100, 100, 100))
             self.selColour = QColor(100, 100, 100)
             self.editName.setText("")
             self.colButton.setIcon(QIcon(pixmap))
-
             self.editName.setEnabled(False)
             self.colButton.setEnabled(False)
             self.saveButton.setEnabled(False)
-
         return
 
     ##
     #  Internal Functions
     ##
 
-    def _addItem(self, key, name, cols, count):
-        """Add a status item to the list.
-        """
+    def _addItem(self, key: str | None, name: str,
+                 cols: tuple[int, int, int], count: int) -> None:
+        """Add a status item to the list."""
         pixmap = QPixmap(self.iPx, self.iPx)
         pixmap.fill(QColor(*cols))
 
@@ -517,9 +515,8 @@ class GuiProjectEditStatus(QWidget):
 
         return
 
-    def _moveItem(self, step):
-        """Move and item up or down step.
-        """
+    def _moveItem(self, step: int) -> None:
+        """Move and item up or down step."""
         selItem = self._getSelectedItem()
         if selItem is None:
             return
@@ -540,17 +537,15 @@ class GuiProjectEditStatus(QWidget):
 
         return
 
-    def _getSelectedItem(self):
-        """Get the currently selected item.
-        """
+    def _getSelectedItem(self) -> QTreeWidgetItem | None:
+        """Get the currently selected item."""
         selItem = self.listBox.selectedItems()
         if len(selItem) > 0:
             return selItem[0]
         return None
 
-    def _usageString(self, nUse):
-        """Generate usage string.
-        """
+    def _usageString(self, nUse: int) -> str:
+        """Generate usage string."""
         if nUse == 0:
             return self.tr("Not in use")
         elif nUse == 1:
@@ -566,17 +561,13 @@ class GuiProjectEditReplace(QWidget):
     COL_KEY  = 0
     COL_REPL = 1
 
-    def __init__(self, projGui):
-        super().__init__(parent=projGui)
+    def __init__(self, parent: QWidget) -> None:
+        super().__init__(parent=parent)
 
-        self.mainConf   = novelwriter.CONFIG
-        self.mainGui    = projGui.mainGui
-        self.mainTheme  = projGui.mainGui.mainTheme
-        self.theProject = projGui.theProject
-        self.arChanged  = False
+        self.arChanged = False
 
-        wCol0 = self.mainConf.pxInt(
-            self.theProject.options.getInt("GuiProjectSettings", "replaceColW", 130)
+        wCol0 = CONFIG.pxInt(
+            SHARED.project.options.getInt("GuiProjectSettings", "replaceColW", 130)
         )
         pageLabel = self.tr("Text Replace List for Preview and Export")
 
@@ -592,7 +583,7 @@ class GuiProjectEditReplace(QWidget):
         self.listBox.setColumnWidth(self.COL_KEY, wCol0)
         self.listBox.setIndentation(0)
 
-        for aKey, aVal in self.theProject.data.autoReplace.items():
+        for aKey, aVal in SHARED.project.data.autoReplace.items():
             newItem = QTreeWidgetItem(["<%s>" % aKey, aVal])
             self.listBox.addTopLevelItem(newItem)
 
@@ -602,10 +593,10 @@ class GuiProjectEditReplace(QWidget):
         # List Controls
         # =============
 
-        self.addButton = QPushButton(self.mainTheme.getIcon("add"), "")
+        self.addButton = QPushButton(SHARED.theme.getIcon("add"), "")
         self.addButton.clicked.connect(self._addEntry)
 
-        self.delButton = QPushButton(self.mainTheme.getIcon("remove"), "")
+        self.delButton = QPushButton(SHARED.theme.getIcon("remove"), "")
         self.delButton.clicked.connect(self._delEntry)
 
         # Edit Form
@@ -652,25 +643,23 @@ class GuiProjectEditReplace(QWidget):
 
         return
 
-    def getNewList(self):
-        """Extract the list from the widget.
-        """
-        newList = {}
+    def getNewList(self) -> dict:
+        """Extract the list from the widget."""
+        new = {}
         for n in range(self.listBox.topLevelItemCount()):
             tItem = self.listBox.topLevelItem(n)
             if tItem is not None:
                 aKey = self._stripNotAllowed(tItem.text(0))
                 aVal = tItem.text(1)
                 if len(aKey) > 0:
-                    newList[aKey] = aVal
-
-        return newList
+                    new[aKey] = aVal
+        return new
 
     ##
     #  Internal Functions
     ##
 
-    def _selectedItem(self):
+    def _selectedItem(self) -> bool:
         """Extract the details from the selected item and populate the
         edit form.
         """
@@ -687,63 +676,53 @@ class GuiProjectEditReplace(QWidget):
         self.editKey.setFocus()
         return True
 
-    def _saveEntry(self):
-        """Save the form data into the list widget.
-        """
+    def _saveEntry(self) -> None:
+        """Save the form data into the list widget."""
         selItem = self._getSelectedItem()
-        if selItem is None:
-            return False
-
-        newKey = self.editKey.text()
-        newVal = self.editValue.text()
-        saveKey = self._stripNotAllowed(newKey)
-
-        if len(saveKey) > 0 and len(newVal) > 0:
-            selItem.setText(self.COL_KEY,  "<%s>" % saveKey)
-            selItem.setText(self.COL_REPL, newVal)
-            self.editKey.clear()
-            self.editValue.clear()
-            self.editKey.setEnabled(False)
-            self.editValue.setEnabled(False)
-            self.listBox.clearSelection()
-            self.arChanged = True
-
+        if selItem:
+            newKey = self.editKey.text()
+            newVal = self.editValue.text()
+            saveKey = self._stripNotAllowed(newKey)
+            if len(saveKey) > 0 and len(newVal) > 0:
+                selItem.setText(self.COL_KEY,  "<%s>" % saveKey)
+                selItem.setText(self.COL_REPL, newVal)
+                self.editKey.clear()
+                self.editValue.clear()
+                self.editKey.setEnabled(False)
+                self.editValue.setEnabled(False)
+                self.listBox.clearSelection()
+                self.arChanged = True
         return
 
-    def _addEntry(self):
-        """Add a new list entry.
-        """
+    def _addEntry(self) -> None:
+        """Add a new list entry."""
         saveKey = "<keyword%d>" % (self.listBox.topLevelItemCount() + 1)
         newVal  = ""
         newItem = QTreeWidgetItem([saveKey, newVal])
         self.listBox.addTopLevelItem(newItem)
-        return True
+        return
 
-    def _delEntry(self):
-        """Delete the selected entry.
-        """
+    def _delEntry(self) -> None:
+        """Delete the selected entry."""
         selItem = self._getSelectedItem()
-        if selItem is None:
-            return False
-        self.listBox.takeTopLevelItem(self.listBox.indexOfTopLevelItem(selItem))
-        self.arChanged = True
-        return True
+        if selItem:
+            self.listBox.takeTopLevelItem(self.listBox.indexOfTopLevelItem(selItem))
+            self.arChanged = True
+        return
 
-    def _getSelectedItem(self):
-        """Extract the currently selected item.
-        """
+    def _getSelectedItem(self) -> QTreeWidgetItem | None:
+        """Extract the currently selected item."""
         selItem = self.listBox.selectedItems()
         if len(selItem) == 0:
             return None
         return selItem[0]
 
-    def _stripNotAllowed(self, theKey):
-        """Clean up the replace key string.
-        """
-        retKey = ""
-        for c in theKey:
+    def _stripNotAllowed(self, key: str) -> str:
+        """Clean up the replace key string."""
+        result = ""
+        for c in key:
             if c.isalnum():
-                retKey += c
-        return retKey
+                result += c
+        return result
 
 # END Class GuiProjectEditReplace
