@@ -1,13 +1,12 @@
 """
 novelWriter – GUI Quotes Dialog
 ===============================
-GUI class for quotes dialog
 
 File History:
-Created: 2020-06-18 [0.9]
+Created: 2020-06-18 [0.9.0] GuiQuoteSelect
 
 This file is a part of novelWriter
-Copyright 2018–2023, Veronica Berglyd Olsen
+Copyright 2018–2024, Veronica Berglyd Olsen
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -22,17 +21,18 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program. If not, see <https://www.gnu.org/licenses/>.
 """
+from __future__ import annotations
 
 import logging
-import novelwriter
 
-from PyQt5.QtGui import QFontMetrics
-from PyQt5.QtCore import Qt, QSize
+from PyQt5.QtGui import QCloseEvent, QFontMetrics
+from PyQt5.QtCore import QSize, Qt, pyqtSlot
 from PyQt5.QtWidgets import (
-    QLabel, QVBoxLayout, QHBoxLayout, QDialog, QDialogButtonBox,
-    QListWidget, QListWidgetItem, QFrame
+    QDialog, QDialogButtonBox, QFrame, QHBoxLayout, QLabel, QListWidget,
+    QListWidgetItem, QVBoxLayout, QWidget
 )
 
+from novelwriter import CONFIG
 from novelwriter.constants import trConst, nwQuotes
 
 logger = logging.getLogger(__name__)
@@ -42,10 +42,13 @@ class GuiQuoteSelect(QDialog):
 
     selectedQuote = ""
 
-    def __init__(self, parent=None, currentQuote='"'):
+    D_KEY = Qt.ItemDataRole.UserRole
+
+    def __init__(self, parent: QWidget, currentQuote: str = '"') -> None:
         super().__init__(parent=parent)
 
-        self.mainConf = novelwriter.CONFIG
+        logger.debug("Create: GuiQuoteSelect")
+        self.setObjectName("GuiQuoteSelect")
 
         self.outerBox = QVBoxLayout()
         self.innerBox = QHBoxLayout()
@@ -77,18 +80,18 @@ class GuiQuoteSelect(QDialog):
             theText = "[ %s ] %s" % (sKey, trConst(sLabel))
             minSize = max(minSize, qMetrics.boundingRect(theText).width())
             qtItem = QListWidgetItem(theText)
-            qtItem.setData(Qt.UserRole, sKey)
+            qtItem.setData(self.D_KEY, sKey)
             self.listBox.addItem(qtItem)
             if sKey == currentQuote:
                 self.listBox.setCurrentItem(qtItem)
 
-        self.listBox.setMinimumWidth(minSize + self.mainConf.pxInt(40))
-        self.listBox.setMinimumHeight(self.mainConf.pxInt(150))
+        self.listBox.setMinimumWidth(minSize + CONFIG.pxInt(40))
+        self.listBox.setMinimumHeight(CONFIG.pxInt(150))
 
         # Buttons
         self.buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
-        self.buttonBox.accepted.connect(self._doAccept)
-        self.buttonBox.rejected.connect(self._doReject)
+        self.buttonBox.accepted.connect(self.accept)
+        self.buttonBox.rejected.connect(self.reject)
 
         # Assemble
         self.labelBox.addWidget(self.previewLabel, 0, Qt.AlignTop)
@@ -102,32 +105,36 @@ class GuiQuoteSelect(QDialog):
 
         self.setLayout(self.outerBox)
 
+        logger.debug("Ready: GuiQuoteSelect")
+
+        return
+
+    def __del__(self) -> None:  # pragma: no cover
+        logger.debug("Delete: GuiQuoteSelect")
         return
 
     ##
-    #  Slots
+    #  Events
     ##
 
-    def _selectedSymbol(self):
-        """Update the preview label and the selected quote style.
-        """
+    def closeEvent(self, event: QCloseEvent) -> None:
+        """Capture the close event and perform cleanup."""
+        event.accept()
+        self.deleteLater()
+        return
+
+    ##
+    #  Private Slots
+    ##
+
+    @pyqtSlot()
+    def _selectedSymbol(self) -> None:
+        """Update the preview label and the selected quote style."""
         selItems = self.listBox.selectedItems()
         if selItems:
-            theSymbol = selItems[0].data(Qt.UserRole)
+            theSymbol = selItems[0].data(self.D_KEY)
             self.previewLabel.setText(theSymbol)
             self.selectedQuote = theSymbol
         return
 
-    def _doAccept(self):
-        """Ok button clicked.
-        """
-        self.accept()
-        return
-
-    def _doReject(self):
-        """Cancel button clicked.
-        """
-        self.reject()
-        return
-
 # END Class GuiQuoteSelect
