@@ -1,7 +1,6 @@
 """
 novelWriter – GUI Novel Tree
 ============================
-GUI classe for the main window novel tree
 
 File History:
 Created: 2020-12-20 [1.1rc1] GuiNovelTree
@@ -9,7 +8,7 @@ Created: 2022-06-12 [2.0rc1] GuiNovelView
 Created: 2022-06-12 [2.0rc1] GuiNovelToolBar
 
 This file is a part of novelWriter
-Copyright 2018–2020, Veronica Berglyd Olsen
+Copyright 2018–2024, Veronica Berglyd Olsen
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,25 +23,31 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program. If not, see <https://www.gnu.org/licenses/>.
 """
+from __future__ import annotations
 
 import logging
-import novelwriter
 
 from enum import Enum
 from time import time
+from typing import TYPE_CHECKING
 
-from PyQt5.QtGui import QFont, QPalette
-from PyQt5.QtCore import Qt, QSize, pyqtSlot, pyqtSignal
+from PyQt5.QtGui import QFocusEvent, QFont, QMouseEvent, QPalette, QResizeEvent
+from PyQt5.QtCore import QModelIndex, QPoint, Qt, QSize, pyqtSlot, pyqtSignal
 from PyQt5.QtWidgets import (
     QAbstractItemView, QActionGroup, QFrame, QHBoxLayout, QHeaderView,
     QInputDialog, QMenu, QSizePolicy, QToolButton, QToolTip, QTreeWidget,
     QTreeWidgetItem, QVBoxLayout, QWidget
 )
 
+from novelwriter import CONFIG, SHARED
 from novelwriter.enum import nwDocMode, nwItemClass, nwOutline
 from novelwriter.common import minmax
 from novelwriter.constants import nwHeaders, nwKeyWords, nwLabels, trConst
-from novelwriter.gui.components import NovelSelector
+from novelwriter.core.index import IndexHeading
+from novelwriter.extensions.novelselector import NovelSelector
+
+if TYPE_CHECKING:  # pragma: no cover
+    from novelwriter.guimain import GuiMain
 
 logger = logging.getLogger(__name__)
 
@@ -63,11 +68,10 @@ class GuiNovelView(QWidget):
     selectedItemChanged = pyqtSignal(str)
     openDocumentRequest = pyqtSignal(str, Enum, str, bool)
 
-    def __init__(self, mainGui):
+    def __init__(self, mainGui: GuiMain) -> None:
         super().__init__(parent=mainGui)
 
-        self.mainGui    = mainGui
-        self.theProject = mainGui.theProject
+        self.mainGui = mainGui
 
         # Build GUI
         self.novelTree = GuiNovelTree(self)
@@ -93,46 +97,41 @@ class GuiNovelView(QWidget):
     #  Methods
     ##
 
-    def updateTheme(self):
-        """Update theme elements.
-        """
+    def updateTheme(self) -> None:
+        """Update theme elements."""
         self.novelBar.updateTheme()
         self.novelTree.updateTheme()
         self.refreshTree()
         return
 
-    def initSettings(self):
-        """Initialise GUI elements that depend on specific settings.
-        """
+    def initSettings(self) -> None:
+        """Initialise GUI elements that depend on specific settings."""
         self.novelTree.initSettings()
         return
 
-    def clearProject(self):
-        """Clear project-related GUI content.
-        """
+    def clearNovelView(self) -> None:
+        """Clear project-related GUI content."""
         self.novelTree.clearContent()
         self.novelBar.clearContent()
         self.novelBar.setEnabled(False)
         return
 
-    def openProjectTasks(self):
-        """Run open project tasks.
-        """
-        lastNovel = self.theProject.data.getLastHandle("novelTree")
-        if lastNovel not in self.theProject.tree:
-            lastNovel = self.theProject.tree.findRoot(nwItemClass.NOVEL)
+    def openProjectTasks(self) -> None:
+        """Run open project tasks."""
+        lastNovel = SHARED.project.data.getLastHandle("novelTree")
+        if lastNovel and lastNovel not in SHARED.project.tree:
+            lastNovel = SHARED.project.tree.findRoot(nwItemClass.NOVEL)
 
         logger.debug("Setting novel tree to root item '%s'", lastNovel)
 
-        lastCol = self.theProject.options.getEnum(
+        lastCol = SHARED.project.options.getEnum(
             "GuiNovelView", "lastCol", NovelTreeColumn, NovelTreeColumn.HIDDEN
         )
-        lastColSize = self.theProject.options.getInt(
+        lastColSize = SHARED.project.options.getInt(
             "GuiNovelView", "lastColSize", 25
         )
 
-        self.clearProject()
-
+        self.clearNovelView()
         self.novelBar.buildNovelRootMenu()
         self.novelBar.setLastColType(lastCol, doRefresh=False)
         self.novelBar.setCurrentRoot(lastNovel)
@@ -142,24 +141,24 @@ class GuiNovelView(QWidget):
 
         return
 
-    def closeProjectTasks(self):
-        """Run closing project tasks.
-        """
+    def closeProjectTasks(self) -> None:
+        """Run closing project tasks."""
         lastColType = self.novelTree.lastColType
         lastColSize = self.novelTree.lastColSize
-        self.theProject.options.setValue("GuiNovelView", "lastCol", lastColType)
-        self.theProject.options.setValue("GuiNovelView", "lastColSize", lastColSize)
+        logger.debug("Saving State: GuiNovelView")
+        pOptions = SHARED.project.options
+        pOptions.setValue("GuiNovelView", "lastCol", lastColType)
+        pOptions.setValue("GuiNovelView", "lastColSize", lastColSize)
+        self.clearNovelView()
         return
 
-    def setTreeFocus(self):
-        """Set the focus to the tree widget.
-        """
+    def setTreeFocus(self) -> None:
+        """Set the focus to the tree widget."""
         self.novelTree.setFocus()
         return
 
-    def treeHasFocus(self):
-        """Check if the novel tree has focus.
-        """
+    def treeHasFocus(self) -> bool:
+        """Check if the novel tree has focus."""
         return self.novelTree.hasFocus()
 
     ##
@@ -167,21 +166,19 @@ class GuiNovelView(QWidget):
     ##
 
     @pyqtSlot()
-    def refreshTree(self):
-        """Refresh the current tree.
-        """
-        self.novelTree.refreshTree(rootHandle=self.theProject.data.getLastHandle("novelTree"))
+    def refreshTree(self) -> None:
+        """Refresh the current tree."""
+        self.novelTree.refreshTree(rootHandle=SHARED.project.data.getLastHandle("novelTree"))
         return
 
     @pyqtSlot(str)
-    def updateRootItem(self, tHandle):
-        """If any root item changes, rebuild the novel root menu.
-        """
+    def updateRootItem(self, tHandle: str) -> None:
+        """If any root item changes, rebuild the novel root menu."""
         self.novelBar.buildNovelRootMenu()
         return
 
     @pyqtSlot(str)
-    def updateNovelItemMeta(self, tHandle):
+    def updateNovelItemMeta(self, tHandle: str) -> None:
         """The meta data of a novel item has changed, and the tree item
         needs to be refreshed.
         """
@@ -193,37 +190,34 @@ class GuiNovelView(QWidget):
 
 class GuiNovelToolBar(QWidget):
 
-    def __init__(self, novelView):
+    def __init__(self, novelView: GuiNovelView) -> None:
         super().__init__(parent=novelView)
 
-        logger.debug("Initialising GuiNovelToolBar ...")
+        logger.debug("Create: GuiNovelToolBar")
 
-        self.mainConf   = novelwriter.CONFIG
-        self.novelView  = novelView
-        self.mainGui    = novelView.mainGui
-        self.theProject = novelView.mainGui.theProject
-        self.mainTheme  = novelView.mainGui.mainTheme
+        self.novelView = novelView
+        self.mainGui   = novelView.mainGui
 
-        iPx = self.mainTheme.baseIconSize
-        mPx = self.mainConf.pxInt(2)
+        iPx = SHARED.theme.baseIconSize
+        mPx = CONFIG.pxInt(2)
 
         self.setContentsMargins(0, 0, 0, 0)
         self.setAutoFillBackground(True)
 
         # Novel Selector
         selFont = self.font()
-        selFont.setWeight(QFont.Bold)
+        selFont.setWeight(QFont.Weight.Bold)
         self.novelPrefix = self.tr("Outline of {0}")
-        self.novelValue = NovelSelector(self, self.theProject, self.mainGui)
+        self.novelValue = NovelSelector(self)
         self.novelValue.setFont(selFont)
-        self.novelValue.setMinimumWidth(self.mainConf.pxInt(150))
-        self.novelValue.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
+        self.novelValue.setMinimumWidth(CONFIG.pxInt(150))
+        self.novelValue.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
         self.novelValue.novelSelectionChanged.connect(self.setCurrentRoot)
 
         self.tbNovel = QToolButton(self)
         self.tbNovel.setToolTip(self.tr("Novel Root"))
         self.tbNovel.setIconSize(QSize(iPx, iPx))
-        self.tbNovel.clicked.connect(self._openNovelSelector)
+        self.tbNovel.clicked.connect(self.novelValue.showPopup)
 
         # Refresh Button
         self.tbRefresh = QToolButton(self)
@@ -232,7 +226,7 @@ class GuiNovelToolBar(QWidget):
         self.tbRefresh.clicked.connect(self._refreshNovelTree)
 
         # More Options Menu
-        self.mMore = QMenu()
+        self.mMore = QMenu(self)
 
         self.mLastCol = self.mMore.addMenu(self.tr("Last Column"))
         self.gLastCol = QActionGroup(self.mMore)
@@ -250,7 +244,7 @@ class GuiNovelToolBar(QWidget):
         self.tbMore.setToolTip(self.tr("More Options"))
         self.tbMore.setIconSize(QSize(iPx, iPx))
         self.tbMore.setMenu(self.mMore)
-        self.tbMore.setPopupMode(QToolButton.InstantPopup)
+        self.tbMore.setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)
 
         # Assemble
         self.outerBox = QHBoxLayout()
@@ -265,7 +259,7 @@ class GuiNovelToolBar(QWidget):
 
         self.updateTheme()
 
-        logger.debug("GuiNovelToolBar initialisation complete")
+        logger.debug("Ready: GuiNovelToolBar")
 
         return
 
@@ -273,16 +267,15 @@ class GuiNovelToolBar(QWidget):
     #  Methods
     ##
 
-    def updateTheme(self):
-        """Update theme elements.
-        """
+    def updateTheme(self) -> None:
+        """Update theme elements."""
         # Icons
-        self.tbNovel.setIcon(self.mainTheme.getIcon("cls_novel"))
-        self.tbRefresh.setIcon(self.mainTheme.getIcon("refresh"))
-        self.tbMore.setIcon(self.mainTheme.getIcon("menu"))
+        self.tbNovel.setIcon(SHARED.theme.getIcon("cls_novel"))
+        self.tbRefresh.setIcon(SHARED.theme.getIcon("refresh"))
+        self.tbMore.setIcon(SHARED.theme.getIcon("menu"))
 
         qPalette = self.palette()
-        qPalette.setBrush(QPalette.Window, qPalette.base())
+        qPalette.setBrush(QPalette.ColorRole.Window, qPalette.base())
         self.setPalette(qPalette)
 
         # StyleSheets
@@ -290,11 +283,12 @@ class GuiNovelToolBar(QWidget):
         buttonStyle = (
             "QToolButton {{padding: {0}px; border: none; background: transparent;}} "
             "QToolButton:hover {{border: none; background: rgba({1},{2},{3},0.2);}}"
-        ).format(self.mainConf.pxInt(2), fadeCol.red(), fadeCol.green(), fadeCol.blue())
+        ).format(CONFIG.pxInt(2), fadeCol.red(), fadeCol.green(), fadeCol.blue())
+        buttonStyleMenu = f"{buttonStyle} QToolButton::menu-indicator {{image: none;}}"
 
         self.tbNovel.setStyleSheet(buttonStyle)
         self.tbRefresh.setStyleSheet(buttonStyle)
-        self.tbMore.setStyleSheet(buttonStyle)
+        self.tbMore.setStyleSheet(buttonStyleMenu)
 
         self.novelValue.setStyleSheet(
             "QComboBox {border-style: none; padding-left: 0;} "
@@ -305,30 +299,26 @@ class GuiNovelToolBar(QWidget):
 
         return
 
-    def clearContent(self):
-        """Run clearing project tasks.
-        """
+    def clearContent(self) -> None:
+        """Run clearing project tasks."""
         self.novelValue.clear()
         self.novelValue.setToolTip("")
         return
 
-    def buildNovelRootMenu(self):
-        """Build the novel root menu.
-        """
+    def buildNovelRootMenu(self) -> None:
+        """Build the novel root menu."""
         self.novelValue.updateList(prefix=self.novelPrefix)
         self.tbNovel.setVisible(self.novelValue.count() > 1)
         return
 
-    def setCurrentRoot(self, rootHandle):
-        """Set the current active root handle.
-        """
+    def setCurrentRoot(self, rootHandle: str | None) -> None:
+        """Set the current active root handle."""
         self.novelValue.setHandle(rootHandle)
         self.novelView.novelTree.refreshTree(rootHandle=rootHandle, overRide=True)
         return
 
-    def setLastColType(self, colType, doRefresh=True):
-        """Set the last column type.
-        """
+    def setLastColType(self, colType: NovelTreeColumn, doRefresh: bool = True) -> None:
+        """Set the last column type."""
         self.aLastCol[colType].setChecked(True)
         self.novelView.novelTree.setLastColType(colType, doRefresh=doRefresh)
         return
@@ -338,24 +328,15 @@ class GuiNovelToolBar(QWidget):
     ##
 
     @pyqtSlot()
-    def _openNovelSelector(self):
-        """Trigger the dropdown list of the novel selector.
-        """
-        self.novelValue.showPopup()
-        return
-
-    @pyqtSlot()
-    def _refreshNovelTree(self):
-        """Rebuild the current tree.
-        """
-        rootHandle = self.theProject.data.getLastHandle("novelTree")
+    def _refreshNovelTree(self) -> None:
+        """Rebuild the current tree."""
+        rootHandle = SHARED.project.data.getLastHandle("novelTree")
         self.novelView.novelTree.refreshTree(rootHandle=rootHandle, overRide=True)
         return
 
     @pyqtSlot()
-    def _selectLastColumnSize(self):
-        """Set the maximum width for the last column.
-        """
+    def _selectLastColumnSize(self) -> None:
+        """Set the maximum width for the last column."""
         oldSize = self.novelView.novelTree.lastColSize
         newSize, isOk = QInputDialog.getInt(
             self, self.tr("Column Size"), self.tr("Maximum column size in %"), oldSize, 15, 75, 5
@@ -369,9 +350,8 @@ class GuiNovelToolBar(QWidget):
     #  Internal Functions
     ##
 
-    def _addLastColAction(self, colType, actionLabel):
-        """Add a column selection entry to the last column menu.
-        """
+    def _addLastColAction(self, colType, actionLabel) -> None:
+        """Add a column selection entry to the last column menu."""
         aLast = self.mLastCol.addAction(actionLabel)
         aLast.setCheckable(True)
         aLast.setActionGroup(self.gLastCol)
@@ -384,25 +364,24 @@ class GuiNovelToolBar(QWidget):
 
 class GuiNovelTree(QTreeWidget):
 
+    C_DATA  = 0
     C_TITLE = 0
     C_WORDS = 1
     C_EXTRA = 2
     C_MORE  = 3
 
-    D_HANDLE = Qt.UserRole
-    D_TITLE  = Qt.UserRole + 1
-    D_KEY    = Qt.UserRole + 2
+    D_HANDLE = Qt.ItemDataRole.UserRole
+    D_TITLE  = Qt.ItemDataRole.UserRole + 1
+    D_KEY    = Qt.ItemDataRole.UserRole + 2
+    D_EXTRA  = Qt.ItemDataRole.UserRole + 3
 
-    def __init__(self, novelView):
+    def __init__(self, novelView: GuiNovelView) -> None:
         super().__init__(parent=novelView)
 
-        logger.debug("Initialising GuiNovelTree ...")
+        logger.debug("Create: GuiNovelTree")
 
-        self.mainConf   = novelwriter.CONFIG
-        self.novelView  = novelView
-        self.mainGui    = novelView.mainGui
-        self.mainTheme  = novelView.mainGui.mainTheme
-        self.theProject = novelView.mainGui.theProject
+        self.novelView = novelView
+        self.mainGui   = novelView.mainGui
 
         # Internal Variables
         self._treeMap     = {}
@@ -419,18 +398,18 @@ class GuiNovelTree(QTreeWidget):
         # Build GUI
         # =========
 
-        iPx = self.mainTheme.baseIconSize
-        cMg = self.mainConf.pxInt(6)
+        iPx = SHARED.theme.baseIconSize
+        cMg = CONFIG.pxInt(6)
 
         self.setIconSize(QSize(iPx, iPx))
-        self.setFrameStyle(QFrame.NoFrame)
+        self.setFrameStyle(QFrame.Shape.NoFrame)
         self.setUniformRowHeights(True)
         self.setAllColumnsShowFocus(True)
         self.setHeaderHidden(True)
         self.setIndentation(0)
         self.setColumnCount(4)
-        self.setSelectionBehavior(QAbstractItemView.SelectRows)
-        self.setSelectionMode(QAbstractItemView.SingleSelection)
+        self.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
+        self.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)
         self.setExpandsOnDoubleClick(False)
         self.setDragEnabled(False)
 
@@ -438,10 +417,10 @@ class GuiNovelTree(QTreeWidget):
         treeHeader = self.header()
         treeHeader.setStretchLastSection(False)
         treeHeader.setMinimumSectionSize(iPx + cMg)
-        treeHeader.setSectionResizeMode(self.C_TITLE, QHeaderView.Stretch)
-        treeHeader.setSectionResizeMode(self.C_WORDS, QHeaderView.ResizeToContents)
-        treeHeader.setSectionResizeMode(self.C_EXTRA, QHeaderView.ResizeToContents)
-        treeHeader.setSectionResizeMode(self.C_MORE, QHeaderView.ResizeToContents)
+        treeHeader.setSectionResizeMode(self.C_TITLE, QHeaderView.ResizeMode.Stretch)
+        treeHeader.setSectionResizeMode(self.C_WORDS, QHeaderView.ResizeMode.ResizeToContents)
+        treeHeader.setSectionResizeMode(self.C_EXTRA, QHeaderView.ResizeMode.ResizeToContents)
+        treeHeader.setSectionResizeMode(self.C_MORE, QHeaderView.ResizeMode.ResizeToContents)
 
         # Pre-Generate Tree Formatting
         fH1 = self.font()
@@ -462,31 +441,29 @@ class GuiNovelTree(QTreeWidget):
         self.initSettings()
         self.updateTheme()
 
-        logger.debug("GuiNovelTree initialisation complete")
+        logger.debug("Ready: GuiNovelTree")
 
         return
 
-    def initSettings(self):
-        """Set or update tree widget settings.
-        """
+    def initSettings(self) -> None:
+        """Set or update tree widget settings."""
         # Scroll bars
-        if self.mainConf.hideVScroll:
-            self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
+        if CONFIG.hideVScroll:
+            self.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
         else:
-            self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
+            self.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
 
-        if self.mainConf.hideHScroll:
-            self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
+        if CONFIG.hideHScroll:
+            self.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
         else:
-            self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
+            self.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
 
         return
 
-    def updateTheme(self):
-        """Update theme elements.
-        """
-        iPx = self.mainTheme.baseIconSize
-        self._pMore = self.mainTheme.loadDecoration("deco_doc_more", pxH=iPx)
+    def updateTheme(self) -> None:
+        """Update theme elements."""
+        iPx = SHARED.theme.baseIconSize
+        self._pMore = SHARED.theme.loadDecoration("deco_doc_more", h=iPx)
         return
 
     ##
@@ -494,34 +471,32 @@ class GuiNovelTree(QTreeWidget):
     ##
 
     @property
-    def lastColType(self):
+    def lastColType(self) -> NovelTreeColumn:
         return self._lastCol
 
     @property
-    def lastColSize(self):
+    def lastColSize(self) -> int:
         return int(self._lastColSize * 100)
 
     ##
     #  Class Methods
     ##
 
-    def clearContent(self):
-        """Clear the GUI content and the related maps.
-        """
+    def clearContent(self) -> None:
+        """Clear the GUI content and the related maps."""
         self.clear()
         self._treeMap = {}
         self._lastBuild = 0
         return
 
-    def refreshTree(self, rootHandle=None, overRide=False):
-        """Refresh the tree if it has been changed.
-        """
+    def refreshTree(self, rootHandle: str | None = None, overRide: bool = False) -> None:
+        """Refresh the tree if it has been changed."""
         logger.debug("Requesting refresh of the novel tree")
         if rootHandle is None:
-            rootHandle = self.theProject.tree.findRoot(nwItemClass.NOVEL)
+            rootHandle = SHARED.project.tree.findRoot(nwItemClass.NOVEL)
 
         treeChanged = self.mainGui.projView.changedSince(self._lastBuild)
-        indexChanged = self.theProject.index.rootChangedSince(rootHandle, self._lastBuild)
+        indexChanged = SHARED.project.index.rootChangedSince(rootHandle, self._lastBuild)
         if not (treeChanged or indexChanged or overRide):
             logger.debug("No changes have been made to the novel index")
             return
@@ -529,108 +504,97 @@ class GuiNovelTree(QTreeWidget):
         selItem = self.selectedItems()
         titleKey = None
         if selItem:
-            titleKey = selItem[0].data(self.C_TITLE, self.D_KEY)
+            titleKey = selItem[0].data(self.C_DATA, self.D_KEY)
 
         self._populateTree(rootHandle)
-        self.theProject.data.setLastHandle(rootHandle, "novelTree")
+        SHARED.project.data.setLastHandle(rootHandle, "novelTree")
 
         if titleKey is not None and titleKey in self._treeMap:
             self._treeMap[titleKey].setSelected(True)
 
         return
 
-    def refreshHandle(self, tHandle):
-        """Refresh the data for a given handle.
-        """
-        idxData = self.theProject.index.getItemData(tHandle)
-        if idxData is None:
-            return
-
-        logger.debug("Refreshing meta data for item '%s'", tHandle)
-        for sTitle, tHeading in idxData.items():
-            sKey = f"{tHandle}:{sTitle}"
-            trItem = self._treeMap.get(sKey, None)
-            if trItem is None:
-                logger.debug("Heading '%s' not in novel tree", sKey)
-                self.refreshTree()
-                return
-
-            self._updateTreeItemValues(trItem, tHeading, tHandle, sTitle)
-
+    def refreshHandle(self, tHandle: str) -> None:
+        """Refresh the data for a given handle."""
+        if idxData := SHARED.project.index.getItemData(tHandle):
+            logger.debug("Refreshing meta data for item '%s'", tHandle)
+            for sTitle, tHeading in idxData.items():
+                sKey = f"{tHandle}:{sTitle}"
+                if trItem := self._treeMap.get(sKey, None):
+                    self._updateTreeItemValues(trItem, tHeading, tHandle, sTitle)
+                else:
+                    logger.debug("Heading '%s' not in novel tree", sKey)
+                    self.refreshTree()
+                    return
         return
 
-    def getSelectedHandle(self):
+    def getSelectedHandle(self) -> tuple[str | None, str | None]:
         """Get the currently selected or active handle. If multiple
         items are selected, return the first.
         """
         selList = self.selectedItems()
         trItem = selList[0] if selList else self.currentItem()
         if isinstance(trItem, QTreeWidgetItem):
-            tHandle = trItem.data(self.C_TITLE, self.D_HANDLE)
-            sTitle = trItem.data(self.C_TITLE, self.D_TITLE)
+            tHandle = trItem.data(self.C_DATA, self.D_HANDLE)
+            sTitle = trItem.data(self.C_DATA, self.D_TITLE)
             return tHandle, sTitle
         return None, None
 
-    def setLastColType(self, colType, doRefresh=True):
-        """Change the content type of the last column and rebuild.
-        """
+    def setLastColType(self, colType: NovelTreeColumn, doRefresh: bool = True) -> None:
+        """Change the content type of the last column and rebuild."""
         if self._lastCol != colType:
             logger.debug("Changing last column to %s", colType.name)
             self._lastCol = colType
             self.setColumnHidden(self.C_EXTRA, colType == NovelTreeColumn.HIDDEN)
             if doRefresh:
-                lastNovel = self.theProject.data.getLastHandle("novelTree")
+                lastNovel = SHARED.project.data.getLastHandle("novelTree")
                 self.refreshTree(rootHandle=lastNovel, overRide=True)
         return
 
-    def setLastColSize(self, colSize):
-        """Set the column size in integer values between 15 and 75.
-        """
+    def setLastColSize(self, colSize: int) -> None:
+        """Set the column size in integer values between 15 and 75."""
         self._lastColSize = minmax(colSize, 15, 75)/100.0
         return
 
-    def setActiveHandle(self, tHandle):
-        """Highlight the rows associated with a given handle.
-        """
-        tStart = time()
-
+    def setActiveHandle(self, tHandle: str | None, doScroll: bool = False) -> None:
+        """Highlight the rows associated with a given handle."""
+        didScroll = False
         self._actHandle = tHandle
         for i in range(self.topLevelItemCount()):
-            tItem = self.topLevelItem(i)
-            if tItem is not None:
-                if tItem.data(self.C_TITLE, self.D_HANDLE) == tHandle:
+            if tItem := self.topLevelItem(i):
+                if tItem.data(self.C_DATA, self.D_HANDLE) == tHandle:
                     tItem.setBackground(self.C_TITLE, self.palette().alternateBase())
                     tItem.setBackground(self.C_WORDS, self.palette().alternateBase())
                     tItem.setBackground(self.C_EXTRA, self.palette().alternateBase())
                     tItem.setBackground(self.C_MORE, self.palette().alternateBase())
+                    if doScroll and not didScroll:
+                        self.scrollToItem(tItem, QAbstractItemView.ScrollHint.PositionAtCenter)
+                        didScroll = True
                 else:
                     tItem.setBackground(self.C_TITLE, self.palette().base())
                     tItem.setBackground(self.C_WORDS, self.palette().base())
                     tItem.setBackground(self.C_EXTRA, self.palette().base())
                     tItem.setBackground(self.C_MORE, self.palette().base())
-
-        logger.debug("Highlighted Novel Tree in %.3f ms", (time() - tStart)*1000)
-
         return
 
     ##
     #  Events
     ##
 
-    def mousePressEvent(self, theEvent):
+    def mousePressEvent(self, event: QMouseEvent) -> None:
         """Overload mousePressEvent to clear selection if clicking the
         mouse in a blank area of the tree view, and to load a document
         for viewing if the user middle-clicked.
         """
-        super().mousePressEvent(theEvent)
+        super().mousePressEvent(event)
 
-        if theEvent.button() == Qt.LeftButton:
-            selItem = self.indexAt(theEvent.pos())
+        if event.button() == Qt.MouseButton.LeftButton:
+            selItem = self.indexAt(event.pos())
             if not selItem.isValid():
                 self.clearSelection()
 
-        elif theEvent.button() == Qt.MiddleButton:
-            selItem = self.itemAt(theEvent.pos())
+        elif event.button() == Qt.MouseButton.MiddleButton:
+            selItem = self.itemAt(event.pos())
             if not isinstance(selItem, QTreeWidgetItem):
                 return
 
@@ -642,16 +606,14 @@ class GuiNovelTree(QTreeWidget):
 
         return
 
-    def focusOutEvent(self, theEvent):
-        """Clear the selection when the tree no longer has focus.
-        """
-        super().focusOutEvent(theEvent)
+    def focusOutEvent(self, event: QFocusEvent) -> None:
+        """Clear the selection when the tree no longer has focus."""
+        super().focusOutEvent(event)
         self.clearSelection()
         return
 
-    def resizeEvent(self, event):
-        """Elide labels in the extra column.
-        """
+    def resizeEvent(self, event: QResizeEvent) -> None:
+        """Elide labels in the extra column."""
         super().resizeEvent(event)
         newW = event.size().width()
         oldW = event.oldSize().width()
@@ -661,8 +623,11 @@ class GuiNovelTree(QTreeWidget):
             for i in range(self.topLevelItemCount()):
                 trItem = self.topLevelItem(i)
                 if isinstance(trItem, QTreeWidgetItem):
-                    lastText = trItem.data(self.C_EXTRA, Qt.UserRole)
-                    trItem.setText(self.C_EXTRA, fMetric.elidedText(lastText, Qt.ElideRight, eliW))
+                    lastText = trItem.data(self.C_DATA, self.D_EXTRA)
+                    trItem.setText(
+                        self.C_EXTRA,
+                        fMetric.elidedText(lastText, Qt.TextElideMode.ElideRight, eliW)
+                    )
         return
 
     ##
@@ -670,18 +635,17 @@ class GuiNovelTree(QTreeWidget):
     ##
 
     @pyqtSlot("QModelIndex")
-    def _treeItemClicked(self, mIndex):
-        """The user clicked on an item in the tree.
-        """
-        if mIndex.column() == self.C_MORE:
-            tHandle = mIndex.siblingAtColumn(self.C_TITLE).data(self.D_HANDLE)
-            sTitle = mIndex.siblingAtColumn(self.C_TITLE).data(self.D_TITLE)
-            tipPos = self.mapToGlobal(self.visualRect(mIndex).topRight())
+    def _treeItemClicked(self, index: QModelIndex) -> None:
+        """The user clicked on an item in the tree."""
+        if index.column() == self.C_MORE:
+            tHandle = index.siblingAtColumn(self.C_DATA).data(self.D_HANDLE)
+            sTitle = index.siblingAtColumn(self.C_DATA).data(self.D_TITLE)
+            tipPos = self.mapToGlobal(self.visualRect(index).topRight())
             self._popMetaBox(tipPos, tHandle, sTitle)
         return
 
     @pyqtSlot()
-    def _treeSelectionChange(self):
+    def _treeSelectionChange(self) -> None:
         """Extract the handle and line number of the currently selected
         title, and send it to the tree meta panel.
         """
@@ -691,7 +655,7 @@ class GuiNovelTree(QTreeWidget):
         return
 
     @pyqtSlot("QTreeWidgetItem*", int)
-    def _treeDoubleClick(self, tItem, colNo):
+    def _treeDoubleClick(self, item: QTreeWidgetItem, column: int) -> None:
         """Extract the handle and line number of the title double-
         clicked, and send it to the main gui class for opening in the
         document editor.
@@ -704,23 +668,22 @@ class GuiNovelTree(QTreeWidget):
     #  Internal Functions
     ##
 
-    def _populateTree(self, rootHandle):
-        """Build the tree based on the project index.
-        """
+    def _populateTree(self, rootHandle: str | None) -> None:
+        """Build the tree based on the project index."""
         self.clearContent()
         tStart = time()
         logger.debug("Building novel tree for root item '%s'", rootHandle)
 
-        novStruct = self.theProject.index.novelStructure(rootHandle=rootHandle, skipExcl=True)
+        novStruct = SHARED.project.index.novelStructure(rootHandle=rootHandle, skipExcl=True)
         for tKey, tHandle, sTitle, novIdx in novStruct:
             if novIdx.level == "H0":
                 continue
 
             newItem = QTreeWidgetItem()
-            newItem.setData(self.C_TITLE, self.D_HANDLE, tHandle)
-            newItem.setData(self.C_TITLE, self.D_TITLE, sTitle)
-            newItem.setData(self.C_TITLE, self.D_KEY, tKey)
-            newItem.setTextAlignment(self.C_WORDS, Qt.AlignRight)
+            newItem.setData(self.C_DATA, self.D_HANDLE, tHandle)
+            newItem.setData(self.C_DATA, self.D_TITLE, sTitle)
+            newItem.setData(self.C_DATA, self.D_KEY, tKey)
+            newItem.setTextAlignment(self.C_WORDS, Qt.AlignmentFlag.AlignRight)
 
             self._updateTreeItemValues(newItem, novIdx, tHandle, sTitle)
             self._treeMap[tKey] = newItem
@@ -733,37 +696,36 @@ class GuiNovelTree(QTreeWidget):
 
         return
 
-    def _updateTreeItemValues(self, trItem, idxItem, tHandle, sTitle):
-        """Set the tree item values from the index entry.
-        """
+    def _updateTreeItemValues(self, trItem: QTreeWidgetItem, idxItem: IndexHeading,
+                              tHandle: str, sTitle: str) -> None:
+        """Set the tree item values from the index entry."""
         iLevel = nwHeaders.H_LEVEL.get(idxItem.level, 0)
-        hDec = self.mainTheme.getHeaderDecoration(iLevel)
+        hDec = SHARED.theme.getHeaderDecoration(iLevel)
 
-        trItem.setData(self.C_TITLE, Qt.DecorationRole, hDec)
+        trItem.setData(self.C_TITLE, Qt.ItemDataRole.DecorationRole, hDec)
         trItem.setText(self.C_TITLE, idxItem.title)
         trItem.setFont(self.C_TITLE, self._hFonts[iLevel])
         trItem.setText(self.C_WORDS, f"{idxItem.wordCount:n}")
-        trItem.setData(self.C_MORE, Qt.DecorationRole, self._pMore)
+        trItem.setData(self.C_MORE, Qt.ItemDataRole.DecorationRole, self._pMore)
 
         # Custom column
         mW = int(self._lastColSize * self.viewport().width())
         lastText, toolTip = self._getLastColumnText(tHandle, sTitle)
-        elideText = self.fontMetrics().elidedText(lastText, Qt.ElideRight, mW)
+        elideText = self.fontMetrics().elidedText(lastText, Qt.TextElideMode.ElideRight, mW)
         trItem.setText(self.C_EXTRA, elideText)
-        trItem.setData(self.C_EXTRA, Qt.UserRole, lastText)
+        trItem.setData(self.C_DATA, self.D_EXTRA, lastText)
         trItem.setToolTip(self.C_EXTRA, toolTip)
 
         return
 
-    def _getLastColumnText(self, tHandle, sTitle):
-        """Generate the text for the last column based on user settings.
-        """
+    def _getLastColumnText(self, tHandle: str, sTitle: str) -> tuple[str, str]:
+        """Generate text for the last column based on user settings."""
         if self._lastCol == NovelTreeColumn.HIDDEN:
             return "", ""
 
         refData = []
         refName = ""
-        theRefs = self.theProject.index.getReferences(tHandle, sTitle)
+        theRefs = SHARED.project.index.getReferences(tHandle, sTitle)
         if self._lastCol == NovelTreeColumn.POV:
             refData = theRefs[nwKeyWords.POV_KEY]
             refName = self._povLabel
@@ -782,14 +744,15 @@ class GuiNovelTree(QTreeWidget):
 
         return "", ""
 
-    def _popMetaBox(self, qPos, tHandle, sTitle):
-        """Show the novel meta data box.
-        """
+    def _popMetaBox(self, qPos: QPoint, tHandle: str, sTitle: str) -> None:
+        """Show the novel meta data box."""
         logger.debug("Generating meta data tooltip for '%s:%s'", tHandle, sTitle)
 
-        pIndex = self.theProject.index
+        pIndex = SHARED.project.index
         novIdx = pIndex.getItemHeader(tHandle, sTitle)
         refTags = pIndex.getReferences(tHandle, sTitle)
+        if not novIdx:
+            return
 
         synopText = novIdx.synopsis
         if synopText:
@@ -819,9 +782,8 @@ class GuiNovelTree(QTreeWidget):
         return
 
     @staticmethod
-    def _appendMetaTag(refs, key, lines):
-        """Generate a reference list for a given reference key.
-        """
+    def _appendMetaTag(refs: dict, key: str, lines: list[str]) -> list[str]:
+        """Generate a reference list for a given reference key."""
         tags = ", ".join(refs.get(key, []))
         if tags:
             lines.append(f"<b>{trConst(nwLabels.KEY_NAME[key])}</b>: {tags}")
