@@ -1,14 +1,13 @@
 """
 novelWriter – Theme and Icons Classes
 =====================================
-Classes managing and caching themes and icons
 
 File History:
 Created: 2019-05-18 [0.1.3] GuiTheme
 Created: 2019-11-08 [0.4]   GuiIcons
 
 This file is a part of novelWriter
-Copyright 2018–2023, Veronica Berglyd Olsen
+Copyright 2018–2024, Veronica Berglyd Olsen
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,11 +22,12 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program. If not, see <https://www.gnu.org/licenses/>.
 """
+from __future__ import annotations
 
 import logging
-import novelwriter
 
 from math import ceil
+from pathlib import Path
 
 from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import qApp
@@ -35,7 +35,8 @@ from PyQt5.QtGui import (
     QPalette, QColor, QIcon, QFont, QFontMetrics, QFontDatabase, QPixmap
 )
 
-from novelwriter.enum import nwItemLayout, nwItemType
+from novelwriter import CONFIG
+from novelwriter.enum import nwItemClass, nwItemLayout, nwItemType
 from novelwriter.error import logException
 from novelwriter.common import NWConfigParser, minmax
 from novelwriter.constants import nwLabels
@@ -50,9 +51,8 @@ logger = logging.getLogger(__name__)
 
 class GuiTheme:
 
-    def __init__(self):
+    def __init__(self) -> None:
 
-        self.mainConf = novelwriter.CONFIG
         self.iconCache = GuiIcons(self)
 
         # Loaded Theme Settings
@@ -97,6 +97,7 @@ class GuiTheme:
         self.colDialD  = [0, 0, 0]
         self.colDialS  = [0, 0, 0]
         self.colHidden = [0, 0, 0]
+        self.colCode   = [0, 0, 0]
         self.colKey    = [0, 0, 0]
         self.colVal    = [0, 0, 0]
         self.colSpell  = [0, 0, 0]
@@ -108,20 +109,19 @@ class GuiTheme:
         # ===========
 
         # Init GUI Font
-        self.guiFontDB = QFontDatabase()
         self._setGuiFont()
 
         # Load Themes
         self._guiPalette  = QPalette()
-        self._themeList   = []
-        self._syntaxList  = []
-        self._availThemes = {}
-        self._availSyntax = {}
+        self._themeList: list[tuple[str, str]] = []
+        self._syntaxList: list[tuple[str, str]] = []
+        self._availThemes: dict[str, Path] = {}
+        self._availSyntax: dict[str, Path] = {}
 
-        self._listConf(self._availSyntax, self.mainConf.assetPath("syntax"))
-        self._listConf(self._availThemes, self.mainConf.assetPath("themes"))
-        self._listConf(self._availSyntax, self.mainConf.dataPath("syntax"))
-        self._listConf(self._availThemes, self.mainConf.dataPath("themes"))
+        self._listConf(self._availSyntax, CONFIG.assetPath("syntax"))
+        self._listConf(self._availThemes, CONFIG.assetPath("themes"))
+        self._listConf(self._availSyntax, CONFIG.dataPath("syntax"))
+        self._listConf(self._availThemes, CONFIG.dataPath("themes"))
 
         self.loadTheme()
         self.loadSyntax()
@@ -130,13 +130,15 @@ class GuiTheme:
         self.getIcon = self.iconCache.getIcon
         self.getPixmap = self.iconCache.getPixmap
         self.getItemIcon = self.iconCache.getItemIcon
+        self.getToggleIcon = self.iconCache.getToggleIcon
         self.loadDecoration = self.iconCache.loadDecoration
         self.getHeaderDecoration = self.iconCache.getHeaderDecoration
+        self.getHeaderDecorationNarrow = self.iconCache.getHeaderDecorationNarrow
 
         # Extract Other Info
         self.guiDPI = qApp.primaryScreen().logicalDotsPerInchX()
         self.guiScale = qApp.primaryScreen().logicalDotsPerInchX()/96.0
-        self.mainConf.guiScale = self.guiScale
+        CONFIG.guiScale = self.guiScale
         logger.debug("GUI DPI: %.1f", self.guiDPI)
         logger.debug("GUI Scale: %.2f", self.guiScale)
 
@@ -168,27 +170,27 @@ class GuiTheme:
     #  Methods
     ##
 
-    def getTextWidth(self, theText, theFont=None):
-        """Returns the width needed to contain a given piece of text.
+    def getTextWidth(self, text: str, font: QFont | None = None) -> int:
+        """Returns the width needed to contain a given piece of text in
+        pixels.
         """
-        if isinstance(theFont, QFont):
-            qMetrics = QFontMetrics(theFont)
+        if isinstance(font, QFont):
+            qMetrics = QFontMetrics(font)
         else:
             qMetrics = QFontMetrics(self.guiFont)
-        return int(ceil(qMetrics.boundingRect(theText).width()))
+        return int(ceil(qMetrics.boundingRect(text).width()))
 
     ##
     #  Theme Methods
     ##
 
-    def loadTheme(self):
-        """Load the currently specified GUI theme.
-        """
-        guiTheme = self.mainConf.guiTheme
+    def loadTheme(self) -> bool:
+        """Load the currently specified GUI theme."""
+        guiTheme = CONFIG.guiTheme
         if guiTheme not in self._availThemes:
             logger.error("Could not find GUI theme '%s'", guiTheme)
             guiTheme = "default"
-            self.mainConf.guiTheme = guiTheme
+            CONFIG.guiTheme = guiTheme
 
         themeFile = self._availThemes.get(guiTheme, None)
         if themeFile is None:
@@ -197,84 +199,85 @@ class GuiTheme:
 
         # Config File
         logger.info("Loading GUI theme '%s'", guiTheme)
-        confParser = NWConfigParser()
+        parser = NWConfigParser()
         try:
             with open(themeFile, mode="r", encoding="utf-8") as inFile:
-                confParser.read_file(inFile)
+                parser.read_file(inFile)
         except Exception:
             logger.error("Could not load theme settings from: %s", themeFile)
             logException()
             return False
 
         # Main
-        cnfSec = "Main"
-        if confParser.has_section(cnfSec):
-            self.themeName        = confParser.rdStr(cnfSec, "name", "")
-            self.themeDescription = confParser.rdStr(cnfSec, "description", "N/A")
-            self.themeAuthor      = confParser.rdStr(cnfSec, "author", "N/A")
-            self.themeCredit      = confParser.rdStr(cnfSec, "credit", "N/A")
-            self.themeUrl         = confParser.rdStr(cnfSec, "url", "")
-            self.themeLicense     = confParser.rdStr(cnfSec, "license", "N/A")
-            self.themeLicenseUrl  = confParser.rdStr(cnfSec, "licenseurl", "")
-            self.themeIcons       = confParser.rdStr(cnfSec, "icontheme", "")
+        sec = "Main"
+        if parser.has_section(sec):
+            self.themeName        = parser.rdStr(sec, "name", "")
+            self.themeDescription = parser.rdStr(sec, "description", "N/A")
+            self.themeAuthor      = parser.rdStr(sec, "author", "N/A")
+            self.themeCredit      = parser.rdStr(sec, "credit", "N/A")
+            self.themeUrl         = parser.rdStr(sec, "url", "")
+            self.themeLicense     = parser.rdStr(sec, "license", "N/A")
+            self.themeLicenseUrl  = parser.rdStr(sec, "licenseurl", "")
+            self.themeIcons       = parser.rdStr(sec, "icontheme", "")
 
         # Palette
-        cnfSec = "Palette"
-        if confParser.has_section(cnfSec):
-            self._setPalette(confParser, cnfSec, "window",          QPalette.Window)
-            self._setPalette(confParser, cnfSec, "windowtext",      QPalette.WindowText)
-            self._setPalette(confParser, cnfSec, "base",            QPalette.Base)
-            self._setPalette(confParser, cnfSec, "alternatebase",   QPalette.AlternateBase)
-            self._setPalette(confParser, cnfSec, "text",            QPalette.Text)
-            self._setPalette(confParser, cnfSec, "tooltipbase",     QPalette.ToolTipBase)
-            self._setPalette(confParser, cnfSec, "tooltiptext",     QPalette.ToolTipText)
-            self._setPalette(confParser, cnfSec, "button",          QPalette.Button)
-            self._setPalette(confParser, cnfSec, "buttontext",      QPalette.ButtonText)
-            self._setPalette(confParser, cnfSec, "brighttext",      QPalette.BrightText)
-            self._setPalette(confParser, cnfSec, "highlight",       QPalette.Highlight)
-            self._setPalette(confParser, cnfSec, "highlightedtext", QPalette.HighlightedText)
-            self._setPalette(confParser, cnfSec, "link",            QPalette.Link)
-            self._setPalette(confParser, cnfSec, "linkvisited",     QPalette.LinkVisited)
+        sec = "Palette"
+        if parser.has_section(sec):
+            self._setPalette(parser, sec, "window",          QPalette.ColorRole.Window)
+            self._setPalette(parser, sec, "windowtext",      QPalette.ColorRole.WindowText)
+            self._setPalette(parser, sec, "base",            QPalette.ColorRole.Base)
+            self._setPalette(parser, sec, "alternatebase",   QPalette.ColorRole.AlternateBase)
+            self._setPalette(parser, sec, "text",            QPalette.ColorRole.Text)
+            self._setPalette(parser, sec, "tooltipbase",     QPalette.ColorRole.ToolTipBase)
+            self._setPalette(parser, sec, "tooltiptext",     QPalette.ColorRole.ToolTipText)
+            self._setPalette(parser, sec, "button",          QPalette.ColorRole.Button)
+            self._setPalette(parser, sec, "buttontext",      QPalette.ColorRole.ButtonText)
+            self._setPalette(parser, sec, "brighttext",      QPalette.ColorRole.BrightText)
+            self._setPalette(parser, sec, "highlight",       QPalette.ColorRole.Highlight)
+            self._setPalette(parser, sec, "highlightedtext", QPalette.ColorRole.HighlightedText)
+            self._setPalette(parser, sec, "link",            QPalette.ColorRole.Link)
+            self._setPalette(parser, sec, "linkvisited",     QPalette.ColorRole.LinkVisited)
         else:
             self._guiPalette = qApp.style().standardPalette()
 
         # GUI
-        cnfSec = "GUI"
-        if confParser.has_section(cnfSec):
-            self.statNone    = self._parseColour(confParser, cnfSec, "statusnone")
-            self.statUnsaved = self._parseColour(confParser, cnfSec, "statusunsaved")
-            self.statSaved   = self._parseColour(confParser, cnfSec, "statussaved")
-
-        # Icons
-        self.iconCache.loadTheme(self.themeIcons or "typicons_light")
+        sec = "GUI"
+        if parser.has_section(sec):
+            self.helpText    = self._parseColour(parser, sec, "helptext")
+            self.statNone    = self._parseColour(parser, sec, "statusnone")
+            self.statUnsaved = self._parseColour(parser, sec, "statusunsaved")
+            self.statSaved   = self._parseColour(parser, sec, "statussaved")
 
         # Update Dependant Colours
         backCol = self._guiPalette.window().color()
         textCol = self._guiPalette.windowText().color()
 
-        backLCol = backCol.lightnessF()
-        textLCol = textCol.lightnessF()
+        backLNess = backCol.lightnessF()
+        textLNess = textCol.lightnessF()
 
-        if backLCol > textLCol:
-            helpLCol = textLCol + 0.65*(backLCol - textLCol)
-        else:
-            helpLCol = backLCol + 0.65*(textLCol - backLCol)
+        if self.helpText == [0, 0, 0]:
+            if backLNess > textLNess:
+                helpLCol = textLNess + 0.35*(backLNess - textLNess)
+            else:
+                helpLCol = backLNess + 0.65*(textLNess - backLNess)
+            self.helpText = [int(255*helpLCol)]*3
 
-        self.helpText = [int(255*helpLCol)]*3
+        # Icons
+        defaultIcons = "typicons_light" if backLNess >= 0.5 else "typicons_dark"
+        self.iconCache.loadTheme(self.themeIcons or defaultIcons)
 
         # Apply Styles
         qApp.setPalette(self._guiPalette)
 
         return True
 
-    def loadSyntax(self):
-        """Load the currently specified syntax highlighter theme.
-        """
-        guiSyntax = self.mainConf.guiSyntax
+    def loadSyntax(self) -> bool:
+        """Load the currently specified syntax highlighter theme."""
+        guiSyntax = CONFIG.guiSyntax
         if guiSyntax not in self._availSyntax:
             logger.error("Could not find syntax theme '%s'", guiSyntax)
             guiSyntax = "default_light"
-            self.mainConf.guiSyntax = guiSyntax
+            CONFIG.guiSyntax = guiSyntax
 
         syntaxFile = self._availSyntax.get(guiSyntax, None)
         if syntaxFile is None:
@@ -316,6 +319,7 @@ class GuiTheme:
             self.colDialD  = self._parseColour(confParser, cnfSec, "doublequotes")
             self.colDialS  = self._parseColour(confParser, cnfSec, "singlequotes")
             self.colHidden = self._parseColour(confParser, cnfSec, "hidden")
+            self.colCode   = self._parseColour(confParser, cnfSec, "shortcode")
             self.colKey    = self._parseColour(confParser, cnfSec, "keyword")
             self.colVal    = self._parseColour(confParser, cnfSec, "value")
             self.colSpell  = self._parseColour(confParser, cnfSec, "spellcheckline")
@@ -325,9 +329,8 @@ class GuiTheme:
 
         return True
 
-    def listThemes(self):
-        """Scan the GUI themes folder and list all themes.
-        """
+    def listThemes(self) -> list[tuple[str, str]]:
+        """Scan the GUI themes folder and list all themes."""
         if self._themeList:
             return self._themeList
 
@@ -342,9 +345,8 @@ class GuiTheme:
 
         return self._themeList
 
-    def listSyntax(self):
-        """Scan the syntax themes folder and list all themes.
-        """
+    def listSyntax(self) -> list[tuple[str, str]]:
+        """Scan the syntax themes folder and list all themes."""
         if self._syntaxList:
             return self._syntaxList
 
@@ -363,30 +365,29 @@ class GuiTheme:
     #  Internal Functions
     ##
 
-    def _setGuiFont(self):
-        """Update the GUI's font style from settings.
-        """
+    def _setGuiFont(self) -> None:
+        """Update the GUI's font style from settings."""
         theFont = QFont()
-        if self.mainConf.guiFont not in self.guiFontDB.families():
-            if self.mainConf.osWindows and "Arial" in self.guiFontDB.families():
+        fontDB = QFontDatabase()
+        if CONFIG.guiFont not in fontDB.families():
+            if CONFIG.osWindows and "Arial" in fontDB.families():
                 # On Windows we default to Arial if possible
                 theFont.setFamily("Arial")
                 theFont.setPointSize(10)
             else:
-                theFont = self.guiFontDB.systemFont(QFontDatabase.GeneralFont)
-            self.mainConf.guiFont = theFont.family()
-            self.mainConf.guiFontSize = theFont.pointSize()
+                theFont = fontDB.systemFont(QFontDatabase.GeneralFont)
+            CONFIG.guiFont = theFont.family()
+            CONFIG.guiFontSize = theFont.pointSize()
         else:
-            theFont.setFamily(self.mainConf.guiFont)
-            theFont.setPointSize(self.mainConf.guiFontSize)
+            theFont.setFamily(CONFIG.guiFont)
+            theFont.setPointSize(CONFIG.guiFontSize)
 
         qApp.setFont(theFont)
 
         return
 
-    def _listConf(self, targetDict, checkDir):
-        """Scan for theme config files and populate the dictionary.
-        """
+    def _listConf(self, targetDict: dict, checkDir: Path) -> bool:
+        """Scan for theme config files and populate the dictionary."""
         if not checkDir.is_dir():
             return False
 
@@ -396,29 +397,28 @@ class GuiTheme:
 
         return True
 
-    def _parseColour(self, confParser, cnfSec, cnfName):
-        """Parse a colour value from a config string.
-        """
-        if confParser.has_option(cnfSec, cnfName):
-            values = confParser.get(cnfSec, cnfName).split(",")
+    def _parseColour(self, parser: NWConfigParser, section: str, name: str) -> list[int]:
+        """Parse a colour value from a config string."""
+        if parser.has_option(section, name):
+            values = parser.get(section, name).split(",")
             result = []
             try:
                 result.append(minmax(int(values[0]), 0, 255))
                 result.append(minmax(int(values[1]), 0, 255))
                 result.append(minmax(int(values[2]), 0, 255))
             except Exception:
-                logger.error("Could not load theme colours for '%s' from config file", cnfName)
+                logger.error("Could not load theme colours for '%s' from config file", name)
                 result = [0, 0, 0]
         else:
-            logger.warning("Could not find theme colours for '%s' in config file", cnfName)
+            logger.warning("Could not find theme colours for '%s' in config file", name)
             result = [0, 0, 0]
         return result
 
-    def _setPalette(self, confParser, cnfSec, cnfName, paletteVal):
-        """Set a palette colour value from a config string.
-        """
+    def _setPalette(self, parser: NWConfigParser, section: str,
+                    name: str, value: QPalette.ColorRole) -> None:
+        """Set a palette colour value from a config string."""
         self._guiPalette.setColor(
-            paletteVal, QColor(*self._parseColour(confParser, cnfSec, cnfName))
+            value, QColor(*self._parseColour(parser, section, name))
         )
         return
 
@@ -444,19 +444,29 @@ class GuiIcons:
     """
 
     ICON_KEYS = {
-        # Project and GUI icons
-        "novelwriter", "cls_archive", "cls_character", "cls_custom", "cls_entity", "cls_none",
-        "cls_novel", "cls_object", "cls_plot", "cls_timeline", "cls_trash", "cls_world",
-        "proj_chapter", "proj_details", "proj_document", "proj_folder", "proj_note", "proj_nwx",
-        "proj_section", "proj_scene", "proj_stats", "proj_title", "search_cancel", "search_case",
-        "search_loop", "search_preserve", "search_project", "search_regex", "search_word",
-        "status_idle", "status_lang", "status_lines", "status_stats", "status_time", "view_build",
-        "view_editor", "view_novel", "view_outline",
+        # Project and GUI Icons
+        "novelwriter", "alert_error", "alert_info", "alert_question", "alert_warn",
+        "build_excluded", "build_filtered", "build_included", "proj_chapter", "proj_details",
+        "proj_document", "proj_folder", "proj_note", "proj_nwx", "proj_section", "proj_scene",
+        "proj_stats", "proj_title", "status_idle", "status_lang", "status_lines", "status_stats",
+        "status_time", "view_build", "view_editor", "view_novel", "view_outline",
+
+        # Class Icons
+        "cls_archive", "cls_character", "cls_custom", "cls_entity", "cls_none", "cls_novel",
+        "cls_object", "cls_plot", "cls_timeline", "cls_trash", "cls_world",
+
+        # Search Icons
+        "search_cancel", "search_case", "search_loop", "search_preserve", "search_project",
+        "search_regex", "search_word",
+
+        # Format Icons
+        "fmt_bold", "fmt_bold-md", "fmt_italic", "fmt_italic-md", "fmt_strike", "fmt_strike-md",
+        "fmt_subscript", "fmt_superscript", "fmt_underline",
 
         # General Button Icons
-        "add", "backward", "bookmark", "checked", "close", "cross", "down", "edit", "forward",
-        "maximise", "menu", "minimise", "noncheckable", "reference", "refresh", "remove",
-        "search_replace", "search", "settings", "unchecked", "up",
+        "add", "backward", "bookmark", "browse", "checked", "close", "cross", "down", "edit",
+        "export", "forward", "maximise", "menu", "minimise", "noncheckable", "panel", "refresh",
+        "remove", "revert", "search_replace", "search", "settings", "unchecked", "up", "view",
 
         # Switches
         "sticky-on", "sticky-off",
@@ -464,25 +474,32 @@ class GuiIcons:
 
         # Decorations
         "deco_doc_h0", "deco_doc_h1", "deco_doc_h2", "deco_doc_h3", "deco_doc_h4", "deco_doc_more",
+        "deco_doc_h0_n", "deco_doc_h1_n", "deco_doc_h2_n", "deco_doc_h3_n", "deco_doc_h4_n",
+        "deco_doc_nt_n",
+    }
+
+    TOGGLE_ICON_KEYS = {
+        "sticky": ("sticky-on", "sticky-off"),
+        "bullet": ("bullet-on", "bullet-off"),
     }
 
     IMAGE_MAP = {
         "wiz-back": "wizard-back.jpg",
     }
 
-    def __init__(self, mainTheme):
+    def __init__(self, mainTheme: GuiTheme) -> None:
 
-        self.mainConf = novelwriter.CONFIG
         self.mainTheme = mainTheme
 
         # Storage
-        self._qIcons    = {}
-        self._themeMap  = {}
-        self._headerDec = []
-        self._confName  = "icons.conf"
+        self._qIcons: dict[str, QIcon] = {}
+        self._themeMap: dict[str, Path] = {}
+        self._headerDec: list[QPixmap] = []
+        self._headerDecNarrow: list[QPixmap] = []
 
         # Icon Theme Path
-        self._iconPath = self.mainConf.assetPath("icons")
+        self._confName = "icons.conf"
+        self._iconPath = CONFIG.assetPath("icons")
 
         # Icon Theme Meta
         self.themeName        = ""
@@ -499,7 +516,7 @@ class GuiIcons:
     #  Actions
     ##
 
-    def loadTheme(self, iconTheme):
+    def loadTheme(self, iconTheme: str) -> bool:
         """Update the theme map. This is more of an init, since many of
         the GUI icons cannot really be replaced without writing specific
         update functions for the classes where they're used.
@@ -507,7 +524,7 @@ class GuiIcons:
         self._themeMap = {}
         themePath = self._iconPath / iconTheme
         if not themePath.is_dir():
-            themePath = self.mainConf.dataPath("icons") / iconTheme
+            themePath = CONFIG.dataPath("icons") / iconTheme
             if not themePath.is_dir():
                 logger.warning("No icons loaded for '%s'", iconTheme)
                 return False
@@ -566,6 +583,7 @@ class GuiIcons:
             self._qIcons[iconKey] = qIcon
 
         self._headerDec = []
+        self._headerDecNarrow = []
 
         return True
 
@@ -573,52 +591,60 @@ class GuiIcons:
     #  Access Functions
     ##
 
-    def loadDecoration(self, decoKey, pxW=None, pxH=None):
+    def loadDecoration(self, name: str, w: int | None = None, h: int | None = None) -> QPixmap:
         """Load graphical decoration element based on the decoration
         map or the icon map. This function always returns a QPixmap.
         """
-        if decoKey in self._themeMap:
-            imgPath = self._themeMap[decoKey]
-        elif decoKey in self.IMAGE_MAP:
-            imgPath = self.mainConf.assetPath("images") / self.IMAGE_MAP[decoKey]
+        if name in self._themeMap:
+            imgPath = self._themeMap[name]
+        elif name in self.IMAGE_MAP:
+            imgPath = CONFIG.assetPath("images") / self.IMAGE_MAP[name]
         else:
-            logger.error("Decoration with name '%s' does not exist", decoKey)
+            logger.error("Decoration with name '%s' does not exist", name)
             return QPixmap()
 
         if not imgPath.is_file():
             logger.error("Asset not found: %s", imgPath)
             return QPixmap()
 
-        theDeco = QPixmap(str(imgPath))
-        if pxW is not None and pxH is not None:
-            return theDeco.scaled(pxW, pxH, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
-        elif pxW is None and pxH is not None:
-            return theDeco.scaledToHeight(pxH, Qt.SmoothTransformation)
-        elif pxW is not None and pxH is None:
-            return theDeco.scaledToWidth(pxW, Qt.SmoothTransformation)
+        pixmap = QPixmap(str(imgPath))
+        if w is not None and h is not None:
+            return pixmap.scaled(w, h, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
+        elif w is None and h is not None:
+            return pixmap.scaledToHeight(h, Qt.SmoothTransformation)
+        elif w is not None and h is None:
+            return pixmap.scaledToWidth(w, Qt.SmoothTransformation)
 
-        return theDeco
+        return pixmap
 
-    def getIcon(self, iconKey):
-        """Return an icon from the icon buffer. If it doesn't exist,
-        return, load it, and if it still doesn't exist, return an empty
-        icon.
-        """
-        if iconKey in self._qIcons:
-            return self._qIcons[iconKey]
+    def getIcon(self, name: str) -> QIcon:
+        """Return an icon from the icon buffer, or load it."""
+        if name in self._qIcons:
+            return self._qIcons[name]
         else:
-            qIcon = self._loadIcon(iconKey)
-            self._qIcons[iconKey] = qIcon
-            return qIcon
+            icon = self._loadIcon(name)
+            self._qIcons[name] = icon
+            return icon
 
-    def getPixmap(self, iconKey, iconSize):
+    def getToggleIcon(self, name: str, size: tuple[int, int]) -> QIcon:
+        """Return a toggle icon from the icon buffer. or load it."""
+        if name in self.TOGGLE_ICON_KEYS:
+            pOne  = self.getPixmap(self.TOGGLE_ICON_KEYS[name][0], size)
+            pTwo = self.getPixmap(self.TOGGLE_ICON_KEYS[name][1], size)
+            icon = QIcon()
+            icon.addPixmap(pOne, QIcon.Normal, QIcon.On)
+            icon.addPixmap(pTwo, QIcon.Normal, QIcon.Off)
+            return icon
+        return QIcon()
+
+    def getPixmap(self, name: str, size: tuple[int, int]) -> QPixmap:
         """Return an icon from the icon buffer as a QPixmap. If it
         doesn't exist, return an empty QPixmap.
         """
-        qIcon = self.getIcon(iconKey)
-        return qIcon.pixmap(iconSize[0], iconSize[1], QIcon.Normal)
+        return self.getIcon(name).pixmap(size[0], size[1], QIcon.Normal)
 
-    def getItemIcon(self, tType, tClass, tLayout, hLevel="H0"):
+    def getItemIcon(self, tType: nwItemType, tClass: nwItemClass,
+                    tLayout: nwItemLayout, hLevel: str = "H0") -> QIcon:
         """Get the correct icon for a project item based on type, class
         and header level
         """
@@ -645,45 +671,58 @@ class GuiIcons:
 
         return self.getIcon(iconName)
 
-    def getHeaderDecoration(self, hLevel):
-        """Get the decoration for a specific header level.
-        """
+    def getHeaderDecoration(self, hLevel: int) -> QPixmap:
+        """Get the decoration for a specific header level."""
         if not self._headerDec:
             iPx = self.mainTheme.baseIconSize
             self._headerDec = [
-                self.loadDecoration("deco_doc_h0", pxH=iPx),
-                self.loadDecoration("deco_doc_h1", pxH=iPx),
-                self.loadDecoration("deco_doc_h2", pxH=iPx),
-                self.loadDecoration("deco_doc_h3", pxH=iPx),
-                self.loadDecoration("deco_doc_h4", pxH=iPx),
+                self.loadDecoration("deco_doc_h0", h=iPx),
+                self.loadDecoration("deco_doc_h1", h=iPx),
+                self.loadDecoration("deco_doc_h2", h=iPx),
+                self.loadDecoration("deco_doc_h3", h=iPx),
+                self.loadDecoration("deco_doc_h4", h=iPx),
             ]
         return self._headerDec[minmax(hLevel, 0, 4)]
 
+    def getHeaderDecorationNarrow(self, hLevel: int) -> QPixmap:
+        """Get the narrow decoration for a specific header level."""
+        if not self._headerDecNarrow:
+            iPx = self.mainTheme.baseIconSize
+            self._headerDecNarrow = [
+                self.loadDecoration("deco_doc_h0_n", h=iPx),
+                self.loadDecoration("deco_doc_h1_n", h=iPx),
+                self.loadDecoration("deco_doc_h2_n", h=iPx),
+                self.loadDecoration("deco_doc_h3_n", h=iPx),
+                self.loadDecoration("deco_doc_h4_n", h=iPx),
+                self.loadDecoration("deco_doc_nt_n", h=iPx),
+            ]
+        return self._headerDecNarrow[minmax(hLevel, 0, 5)]
+
     ##
     #  Internal Functions
     ##
 
-    def _loadIcon(self, iconKey):
+    def _loadIcon(self, name: str) -> QIcon:
         """Load an icon from the assets themes folder. Is guaranteed to
         return a QIcon.
         """
-        if iconKey not in self.ICON_KEYS:
-            logger.error("Requested unknown icon name '%s'", iconKey)
+        if name not in self.ICON_KEYS:
+            logger.error("Requested unknown icon name '%s'", name)
             return QIcon()
 
         # If we just want the app icons, return right away
-        if iconKey == "novelwriter":
+        if name == "novelwriter":
             return QIcon(str(self._iconPath / "novelwriter.svg"))
-        elif iconKey == "proj_nwx":
+        elif name == "proj_nwx":
             return QIcon(str(self._iconPath / "x-novelwriter-project.svg"))
 
         # Otherwise, we load from the theme folder
-        if iconKey in self._themeMap:
-            logger.debug("Loading: %s", self._themeMap[iconKey].name)
-            return QIcon(str(self._themeMap[iconKey]))
+        if name in self._themeMap:
+            logger.debug("Loading: %s", self._themeMap[name].name)
+            return QIcon(str(self._themeMap[name]))
 
         # If we didn't find one, give up and return an empty icon
-        logger.warning("Did not load an icon for '%s'", iconKey)
+        logger.warning("Did not load an icon for '%s'", name)
 
         return QIcon()
 
@@ -694,9 +733,8 @@ class GuiIcons:
 #  Module Functions
 # =============================================================================================== #
 
-def _loadInternalName(confParser, confFile):
-    """Open a conf file and read the 'name' setting.
-    """
+def _loadInternalName(confParser: NWConfigParser, confFile: str | Path) -> str:
+    """Open a conf file and read the 'name' setting."""
     try:
         with open(confFile, mode="r", encoding="utf-8") as inFile:
             confParser.read_file(inFile)
