@@ -1,13 +1,12 @@
 """
 novelWriter – GUI Writing Statistics
 ====================================
-GUI class for the session statistics dialog
 
 File History:
-Created: 2019-10-20 [0.3]
+Created: 2019-10-20 [0.3.0] GuiWritingStats
 
 This file is a part of novelWriter
-Copyright 2018–2023, Veronica Berglyd Olsen
+Copyright 2018–2024, Veronica Berglyd Olsen
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -22,31 +21,38 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program. If not, see <https://www.gnu.org/licenses/>.
 """
+from __future__ import annotations
 
 import json
 import logging
-import novelwriter
 
-from pathlib import Path
 from datetime import datetime
+from typing import TYPE_CHECKING
 
-from PyQt5.QtGui import QPixmap, QCursor
-from PyQt5.QtCore import Qt
+from PyQt5.QtGui import QCloseEvent, QPixmap, QCursor
+from PyQt5.QtCore import Qt, pyqtSlot
 from PyQt5.QtWidgets import (
     qApp, QDialog, QTreeWidget, QTreeWidgetItem, QDialogButtonBox, QGridLayout,
     QLabel, QGroupBox, QMenu, QAction, QFileDialog, QSpinBox, QHBoxLayout
 )
 
-from novelwriter.enum import nwAlert
+from novelwriter import CONFIG, SHARED
 from novelwriter.error import formatException
 from novelwriter.common import formatTime, checkInt, checkIntTuple, minmax
-from novelwriter.custom import QSwitch
-from novelwriter.constants import nwConst, nwFiles
+from novelwriter.constants import nwConst
+from novelwriter.extensions.switch import NSwitch
+
+if TYPE_CHECKING:  # pragma: no cover
+    from novelwriter.guimain import GuiMain
 
 logger = logging.getLogger(__name__)
 
 
 class GuiWritingStats(QDialog):
+    """GUI Tools: Writing Statistics
+
+    Displays data from the NWSessionLog object.
+    """
 
     C_TIME   = 0
     C_LENGTH = 1
@@ -57,43 +63,40 @@ class GuiWritingStats(QDialog):
     FMT_JSON = 0
     FMT_CSV  = 1
 
-    def __init__(self, mainGui):
+    def __init__(self, mainGui: GuiMain) -> None:
         super().__init__(parent=mainGui)
 
-        logger.debug("Initialising GuiWritingStats ...")
+        logger.debug("Create: GuiWritingStats")
         self.setObjectName("GuiWritingStats")
-
-        self.mainConf   = novelwriter.CONFIG
-        self.mainGui    = mainGui
-        self.mainTheme  = mainGui.mainTheme
-        self.theProject = mainGui.theProject
+        if CONFIG.osDarwin:
+            self.setWindowFlag(Qt.WindowType.Tool)
 
         self.logData    = []
         self.filterData = []
         self.timeFilter = 0.0
         self.wordOffset = 0
 
-        pOptions = self.theProject.options
+        pOptions = SHARED.project.options
 
         self.setWindowTitle(self.tr("Writing Statistics"))
-        self.setMinimumWidth(self.mainConf.pxInt(420))
-        self.setMinimumHeight(self.mainConf.pxInt(400))
+        self.setMinimumWidth(CONFIG.pxInt(420))
+        self.setMinimumHeight(CONFIG.pxInt(400))
         self.resize(
-            self.mainConf.pxInt(pOptions.getInt("GuiWritingStats", "winWidth",  550)),
-            self.mainConf.pxInt(pOptions.getInt("GuiWritingStats", "winHeight", 500))
+            CONFIG.pxInt(pOptions.getInt("GuiWritingStats", "winWidth",  550)),
+            CONFIG.pxInt(pOptions.getInt("GuiWritingStats", "winHeight", 500))
         )
 
         # List Box
-        wCol0 = self.mainConf.pxInt(
+        wCol0 = CONFIG.pxInt(
             pOptions.getInt("GuiWritingStats", "widthCol0", 180)
         )
-        wCol1 = self.mainConf.pxInt(
+        wCol1 = CONFIG.pxInt(
             pOptions.getInt("GuiWritingStats", "widthCol1", 80)
         )
-        wCol2 = self.mainConf.pxInt(
+        wCol2 = CONFIG.pxInt(
             pOptions.getInt("GuiWritingStats", "widthCol2", 80)
         )
-        wCol3 = self.mainConf.pxInt(
+        wCol3 = CONFIG.pxInt(
             pOptions.getInt("GuiWritingStats", "widthCol3", 80)
         )
 
@@ -122,12 +125,12 @@ class GuiWritingStats(QDialog):
             pOptions.getInt("GuiWritingStats", "sortOrder", Qt.DescendingOrder),
             (Qt.AscendingOrder, Qt.DescendingOrder), Qt.DescendingOrder
         )
-        self.listBox.sortByColumn(sortCol, sortOrder)
+        self.listBox.sortByColumn(sortCol, sortOrder)  # type: ignore
         self.listBox.setSortingEnabled(True)
 
         # Word Bar
-        self.barHeight = int(round(0.5*self.mainTheme.fontPixelSize))
-        self.barWidth = self.mainConf.pxInt(200)
+        self.barHeight = int(round(0.5*SHARED.theme.fontPixelSize))
+        self.barWidth = CONFIG.pxInt(200)
         self.barImage = QPixmap(self.barHeight, self.barHeight)
         self.barImage.fill(self.palette().highlight().color())
 
@@ -137,27 +140,27 @@ class GuiWritingStats(QDialog):
         self.infoBox.setLayout(self.infoForm)
 
         self.labelTotal = QLabel(formatTime(0))
-        self.labelTotal.setFont(self.mainTheme.guiFontFixed)
+        self.labelTotal.setFont(SHARED.theme.guiFontFixed)
         self.labelTotal.setAlignment(Qt.AlignVCenter | Qt.AlignRight)
 
         self.labelIdleT = QLabel(formatTime(0))
-        self.labelIdleT.setFont(self.mainTheme.guiFontFixed)
+        self.labelIdleT.setFont(SHARED.theme.guiFontFixed)
         self.labelIdleT.setAlignment(Qt.AlignVCenter | Qt.AlignRight)
 
         self.labelFilter = QLabel(formatTime(0))
-        self.labelFilter.setFont(self.mainTheme.guiFontFixed)
+        self.labelFilter.setFont(SHARED.theme.guiFontFixed)
         self.labelFilter.setAlignment(Qt.AlignVCenter | Qt.AlignRight)
 
         self.novelWords = QLabel("0")
-        self.novelWords.setFont(self.mainTheme.guiFontFixed)
+        self.novelWords.setFont(SHARED.theme.guiFontFixed)
         self.novelWords.setAlignment(Qt.AlignVCenter | Qt.AlignRight)
 
         self.notesWords = QLabel("0")
-        self.notesWords.setFont(self.mainTheme.guiFontFixed)
+        self.notesWords.setFont(SHARED.theme.guiFontFixed)
         self.notesWords.setAlignment(Qt.AlignVCenter | Qt.AlignRight)
 
         self.totalWords = QLabel("0")
-        self.totalWords.setFont(self.mainTheme.guiFontFixed)
+        self.totalWords.setFont(SHARED.theme.guiFontFixed)
         self.totalWords.setAlignment(Qt.AlignVCenter | Qt.AlignRight)
 
         lblTTime   = QLabel(self.tr("Total Time:"))
@@ -184,43 +187,43 @@ class GuiWritingStats(QDialog):
         self.infoForm.setRowStretch(6, 1)
 
         # Filter Options
-        sPx = self.mainTheme.baseIconSize
+        sPx = SHARED.theme.baseIconSize
 
         self.filterBox = QGroupBox(self.tr("Filters"), self)
         self.filterForm = QGridLayout(self)
         self.filterBox.setLayout(self.filterForm)
 
-        self.incNovel = QSwitch(width=2*sPx, height=sPx)
+        self.incNovel = NSwitch(width=2*sPx, height=sPx)
         self.incNovel.setChecked(
             pOptions.getBool("GuiWritingStats", "incNovel", True)
         )
         self.incNovel.clicked.connect(self._updateListBox)
 
-        self.incNotes = QSwitch(width=2*sPx, height=sPx)
+        self.incNotes = NSwitch(width=2*sPx, height=sPx)
         self.incNotes.setChecked(
             pOptions.getBool("GuiWritingStats", "incNotes", True)
         )
         self.incNotes.clicked.connect(self._updateListBox)
 
-        self.hideZeros = QSwitch(width=2*sPx, height=sPx)
+        self.hideZeros = NSwitch(width=2*sPx, height=sPx)
         self.hideZeros.setChecked(
             pOptions.getBool("GuiWritingStats", "hideZeros", True)
         )
         self.hideZeros.clicked.connect(self._updateListBox)
 
-        self.hideNegative = QSwitch(width=2*sPx, height=sPx)
+        self.hideNegative = NSwitch(width=2*sPx, height=sPx)
         self.hideNegative.setChecked(
             pOptions.getBool("GuiWritingStats", "hideNegative", False)
         )
         self.hideNegative.clicked.connect(self._updateListBox)
 
-        self.groupByDay = QSwitch(width=2*sPx, height=sPx)
+        self.groupByDay = NSwitch(width=2*sPx, height=sPx)
         self.groupByDay.setChecked(
             pOptions.getBool("GuiWritingStats", "groupByDay", False)
         )
         self.groupByDay.clicked.connect(self._updateListBox)
 
-        self.showIdleTime = QSwitch(width=2*sPx, height=sPx)
+        self.showIdleTime = NSwitch(width=2*sPx, height=sPx)
         self.showIdleTime.setChecked(
             pOptions.getBool("GuiWritingStats", "showIdleTime", False)
         )
@@ -287,13 +290,16 @@ class GuiWritingStats(QDialog):
 
         self.setLayout(self.outerBox)
 
-        logger.debug("GuiWritingStats initialisation complete")
+        logger.debug("Ready: GuiWritingStats")
 
         return
 
-    def populateGUI(self):
-        """Populate list box with data from the log file.
-        """
+    def __del__(self) -> None:  # pragma: no cover
+        logger.debug("Delete: GuiWritingStats")
+        return
+
+    def populateGUI(self) -> None:
+        """Populate list box with data from the log file."""
         qApp.setOverrideCursor(QCursor(Qt.WaitCursor))
         self._loadLogFile()
         self._updateListBox()
@@ -301,20 +307,30 @@ class GuiWritingStats(QDialog):
         return
 
     ##
-    #  Slots
+    #  Events
     ##
 
-    def _doClose(self):
-        """Save the state of the window, clear cache, end close.
-        """
+    def closeEvent(self, event: QCloseEvent) -> None:
+        """Capture the user closing the window."""
+        event.accept()
+        self.deleteLater()
+        return
+
+    ##
+    #  Private Slots
+    ##
+
+    @pyqtSlot()
+    def _doClose(self) -> None:
+        """Save the state of the window, clear cache, end close."""
         self.logData = []
 
-        winWidth     = self.mainConf.rpxInt(self.width())
-        winHeight    = self.mainConf.rpxInt(self.height())
-        widthCol0    = self.mainConf.rpxInt(self.listBox.columnWidth(0))
-        widthCol1    = self.mainConf.rpxInt(self.listBox.columnWidth(1))
-        widthCol2    = self.mainConf.rpxInt(self.listBox.columnWidth(2))
-        widthCol3    = self.mainConf.rpxInt(self.listBox.columnWidth(3))
+        winWidth     = CONFIG.rpxInt(self.width())
+        winHeight    = CONFIG.rpxInt(self.height())
+        widthCol0    = CONFIG.rpxInt(self.listBox.columnWidth(0))
+        widthCol1    = CONFIG.rpxInt(self.listBox.columnWidth(1))
+        widthCol2    = CONFIG.rpxInt(self.listBox.columnWidth(2))
+        widthCol3    = CONFIG.rpxInt(self.listBox.columnWidth(3))
         sortCol      = self.listBox.sortColumn()
         sortOrder    = self.listBox.header().sortIndicatorOrder()
         incNovel     = self.incNovel.isChecked()
@@ -325,7 +341,8 @@ class GuiWritingStats(QDialog):
         showIdleTime = self.showIdleTime.isChecked()
         histMax      = self.histMax.value()
 
-        pOptions = self.theProject.options
+        logger.debug("Saving State: GuiWritingStats")
+        pOptions = SHARED.project.options
         pOptions.setValue("GuiWritingStats", "winWidth",     winWidth)
         pOptions.setValue("GuiWritingStats", "winHeight",    winHeight)
         pOptions.setValue("GuiWritingStats", "widthCol0",    widthCol0)
@@ -342,13 +359,13 @@ class GuiWritingStats(QDialog):
         pOptions.setValue("GuiWritingStats", "showIdleTime", showIdleTime)
         pOptions.setValue("GuiWritingStats", "histMax",      histMax)
         pOptions.saveSettings()
+
         self.close()
 
         return
 
-    def _saveData(self, dataFmt):
-        """Save the content of the list box to a file.
-        """
+    def _saveData(self, dataFmt: int) -> bool:
+        """Save the content of the list box to a file."""
         fileExt = ""
         textFmt = ""
 
@@ -362,14 +379,14 @@ class GuiWritingStats(QDialog):
             return False
 
         # Generate the file name
-        savePath = self.mainConf.lastPath() / f"sessionStats.{fileExt}"
+        savePath = CONFIG.lastPath() / f"sessionStats.{fileExt}"
         savePath, _ = QFileDialog.getSaveFileName(
             self, self.tr("Save Data As"), str(savePath), "%s (*.%s)" % (textFmt, fileExt)
         )
         if not savePath:
             return False
 
-        self.mainConf.setLastPath(savePath)
+        CONFIG.setLastPath(savePath)
 
         # Do the actual writing
         wSuccess = False
@@ -406,13 +423,15 @@ class GuiWritingStats(QDialog):
 
         # Report to user
         if wSuccess:
-            self.mainGui.makeAlert([
-                self.tr("{0} file successfully written to:").format(textFmt), savePath
-            ], nwAlert.INFO)
+            SHARED.info(
+                self.tr("{0} file successfully written to:").format(textFmt),
+                info=savePath
+            )
         else:
-            self.mainGui.makeAlert([
-                self.tr("Failed to write {0} file.").format(textFmt), errMsg
-            ], nwAlert.ERROR)
+            SHARED.error(
+                self.tr("Failed to write {0} file.").format(textFmt),
+                info=errMsg
+            )
 
         return wSuccess
 
@@ -420,9 +439,8 @@ class GuiWritingStats(QDialog):
     #  Internal Functions
     ##
 
-    def _loadLogFile(self):
-        """Load the content of the log file into a buffer.
-        """
+    def _loadLogFile(self) -> None:
+        """Load the content of the log file into a buffer."""
         logger.debug("Loading session log file")
 
         self.logData = []
@@ -433,50 +451,30 @@ class GuiWritingStats(QDialog):
         ttTime = 0
         ttIdle = 0
 
-        logFile = self.theProject.storage.getMetaFile(nwFiles.SESS_STATS)
-        if not isinstance(logFile, Path) or not logFile.exists():
-            logger.info("This project has no writing stats logfile")
-            return False
+        for record in SHARED.project.session.iterRecords():
+            rType = record.get("type")
+            if rType == "initial":
+                self.wordOffset = checkInt(record.get("offset"), 0)
+                logger.debug("Initial word count when log was started is %d" % self.wordOffset)
+            elif rType == "record":
+                try:
+                    dStart = datetime.fromisoformat(str(record.get("start")))
+                    dEnd   = datetime.fromisoformat(str(record.get("end")))
+                except Exception:
+                    logger.error("Invalid session log record")
+                    continue
+                wcNovel = max(checkInt(record.get("novel"), 0), 0)
+                wcNotes = max(checkInt(record.get("notes"), 0), 0)
+                sIdle = max(checkInt(record.get("idle"), 0), 0)
 
-        try:
-            with open(logFile, mode="r", encoding="utf-8") as inFile:
-                for inLine in inFile:
-                    if inLine.startswith("#"):
-                        if inLine.startswith("# Offset"):
-                            self.wordOffset = checkInt(inLine[9:].strip(), 0)
-                            logger.debug(
-                                "Initial word count when log was started is %d" % self.wordOffset
-                            )
-                        continue
+                tDiff = dEnd - dStart
+                sDiff = tDiff.total_seconds()
+                ttTime += sDiff
+                ttIdle += sIdle
+                ttNovel = wcNovel
+                ttNotes = wcNotes
 
-                    inData = inLine.split()
-                    if len(inData) < 6:
-                        continue
-
-                    dStart = datetime.fromisoformat(" ".join(inData[0:2]))
-                    dEnd   = datetime.fromisoformat(" ".join(inData[2:4]))
-
-                    sIdle = 0
-                    if len(inData) > 6:
-                        sIdle = checkInt(inData[6], 0)
-
-                    tDiff = dEnd - dStart
-                    sDiff = tDiff.total_seconds()
-                    ttTime += sDiff
-                    ttIdle += sIdle
-
-                    wcNovel = int(inData[4])
-                    wcNotes = int(inData[5])
-                    ttNovel = wcNovel
-                    ttNotes = wcNotes
-
-                    self.logData.append((dStart, sDiff, wcNovel, wcNotes, sIdle))
-
-        except Exception as exc:
-            self.mainGui.makeAlert(self.tr(
-                "Failed to read session log file."
-            ), nwAlert.ERROR, exception=exc)
-            return False
+                self.logData.append((dStart, sDiff, wcNovel, wcNotes, sIdle))
 
         ttWords = ttNovel + ttNotes
         self.labelTotal.setText(formatTime(round(ttTime)))
@@ -485,15 +483,15 @@ class GuiWritingStats(QDialog):
         self.notesWords.setText(f"{ttNotes:n}")
         self.totalWords.setText(f"{ttWords:n}")
 
-        return True
+        return
 
     ##
-    #  Slots
+    #  Private Slots
     ##
 
-    def _updateListBox(self):
-        """Load/reload the content of the list box.
-        """
+    @pyqtSlot()
+    def _updateListBox(self) -> None:
+        """Load/reload the content of the list box."""
         self.listBox.clear()
         self.timeFilter = 0.0
 
@@ -600,19 +598,16 @@ class GuiWritingStats(QDialog):
             newItem.setTextAlignment(self.C_COUNT, Qt.AlignRight)
             newItem.setTextAlignment(self.C_BAR, Qt.AlignLeft | Qt.AlignVCenter)
 
-            newItem.setFont(self.C_TIME, self.mainTheme.guiFontFixed)
-            newItem.setFont(self.C_LENGTH, self.mainTheme.guiFontFixed)
-            newItem.setFont(self.C_COUNT, self.mainTheme.guiFontFixed)
-            if showIdleTime:
-                newItem.setFont(self.C_IDLE, self.mainTheme.guiFontFixed)
-            else:
-                newItem.setFont(self.C_IDLE, self.mainTheme.guiFont)
+            newItem.setFont(self.C_TIME, SHARED.theme.guiFontFixed)
+            newItem.setFont(self.C_LENGTH, SHARED.theme.guiFontFixed)
+            newItem.setFont(self.C_COUNT, SHARED.theme.guiFontFixed)
+            newItem.setFont(self.C_IDLE, SHARED.theme.guiFontFixed)
 
             self.listBox.addTopLevelItem(newItem)
             self.timeFilter += sDiff
 
         self.labelFilter.setText(formatTime(round(self.timeFilter)))
 
-        return True
+        return
 
 # END Class GuiWritingStats
